@@ -8,12 +8,13 @@
 #include <string.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <ctype.h>	/* For isspace() */
 
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #include <unistd.h>
 #define direct dirent
-#else
+#elif !defined(_MSC_VER)
 #include <sys/dir.h>
 #endif
 
@@ -21,20 +22,30 @@
 #include <errno.h>
 #include <limits.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
-
+#endif
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #include "Xw/WorkSpace.h"
 #include "Xw/TextEdit.h"
 #include "Xw/Toggle.h"
 #endif
+#endif
+
+#if defined(XC_WIN32) && defined(TCL_WRAPPER)
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/cursorfont.h>
+#endif
+
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -79,6 +90,8 @@ GC	 hgc = NULL, sgc = NULL;
 char	 *cwdname = NULL;
 fileliststruct *files;
 
+#if !defined(XC_WIN32) || defined(TCL_WRAPPER)
+
 /*-------------------------------------------------------------------------*/
 /* Compare two filenames (for use by qsort())				   */
 /*-------------------------------------------------------------------------*/
@@ -173,6 +186,8 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
    XSetFunction(dpy, areawin->gc, areawin->gctype);
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Read a crash file to find the name of the original file.		*/
 /*----------------------------------------------------------------------*/
@@ -220,7 +235,7 @@ void crashrecover()
    }
    xobjs.tempfile = strdup(_STR2);
 
-   startloadfile();
+   startloadfile(-1);
 }
 
 /*----------------------------------------------------------------------*/
@@ -232,7 +247,9 @@ void findcrashfiles()
    DIR *cwd;
    struct direct *dp;
    struct stat sbuf;
+#ifndef _MSC_VER
    uid_t userid = getuid();
+#endif
    time_t recent = 0;
    char *snptr;
    int pid;
@@ -252,16 +269,22 @@ void findcrashfiles()
 	       pid = -1;
 	    *dotptr = '.';
          }
-         if ((!stat(_STR, &sbuf)) && (sbuf.st_uid == userid)) {
+         if ((!stat(_STR, &sbuf))
+#ifndef _MSC_VER
+	     && (sbuf.st_uid == userid)
+#endif
+	     ) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
 
 	       /* Check if the PID belongs to an active process	*/
 	       /* by sending a CONT signal and checking if 	*/
 	       /* there was no error result.			*/
 
+#ifndef _MSC_VER
 	       if (pid != -1)
 		  if (kill((pid_t)pid, SIGCONT) == 0)
 		     continue;
+#endif
 
 	       recent = sbuf.st_ctime;
 	       strcpy(_STR2, _STR);
@@ -295,17 +318,40 @@ void findcrashfiles()
 }  
 
 /*----------------------------------------------------------------------*/
-/* Match a filename extension against the file filter string.		*/
+/* Match a filename extension against the file filter list.		*/
 /*----------------------------------------------------------------------*/
 
+#if !defined(XC_WIN32) || defined(TCL_WRAPPER)
+
 Boolean match_filter(char *fname, char *filter)
 {
    char *dotptr = strrchr(fname, '.');
+   char *filtptr, *endptr;
+   int filtlen, extlen;
 
-   if ((dotptr != NULL) && (filter != NULL) &&
-       (filter[0] != '\0') && (strstr(dotptr, filter) != NULL))
-      return True;
+   if (filter == NULL) return False;
+   if (dotptr == NULL) return False;
 
+   /* New 11/08:  Empty string for filter is a wildcard match, the same */
+   /* as turning off the filter.					*/
+
+   if (filter[0] == '\0') return True;
+
+   dotptr++;
+   extlen = strlen(dotptr);
+   filtptr = filter;
+
+   while (*filtptr != '\0') {
+      endptr = filtptr;
+      while (*endptr != '\0' && !isspace(*endptr)) endptr++;
+      filtlen = (int)(endptr - filtptr);
+      if (filtlen == extlen)
+         if (!strncmp(dotptr, filtptr, extlen))
+	    return True;
+
+      filtptr = endptr;
+      while (*filtptr != '\0' && isspace(*filtptr)) filtptr++;
+   }
    return False;
 }
 
@@ -316,7 +362,9 @@ Boolean match_filter(char *fname, char *filter)
 void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
    XGCValues	values;
+#ifndef TCL_WRAPPER
    Arg wargs[2];
+#endif
    DIR *cwd;
    char *filter;
    Window lwin = xcWindow(w);
@@ -464,12 +512,11 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 void newfilelist(xcWidget w, popupstruct *okaystruct)
 {
    short n;
-   xcWidget textw = okaystruct->textw;
 
 #ifdef TCL_WRAPPER
    int bval;
    int result;
-   char *cstr = (char *)Tcl_GetVar2(xcinterp, "XCOps", "filter", TCL_NAMESPACE_ONLY);
+   char *cstr = (char *)Tcl_GetVar2(xcinterp, "XCOps", "filter", 0);
    if (cstr == NULL) {
       Wprintf("Error: No variable $XCOps(filter) in Tcl!");
       return;
@@ -480,6 +527,8 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
       return;  
    }
    xobjs.filefilter = bval;
+#else
+   xcWidget textw = okaystruct->textw;
 #endif
 
    for (n = 0; n < flfiles; n++)
@@ -509,24 +558,26 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
 
 void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
-   Arg wargs[2];
    Window lwin = xcWindow(w);
    Dimension textwidth, textheight;
-   short filenum, n = 0;
+   short filenum;
    char *tbuf, *ebuf;
-   xcWidget textw = okaystruct->textw;
-
-   flcurrent = -1;
 
 #ifdef TCL_WRAPPER
    textwidth = Tk_Width(w);
    textheight = Tk_Height(w);
 #else
+   Arg wargs[2];
+   short n = 0;
+   xcWidget textw = okaystruct->textw;
+
    XtnSetArg(XtNwidth, &textwidth);
    XtnSetArg(XtNheight, &textheight);
    XtGetValues(w, wargs, n);
 #endif
 
+   flcurrent = -1;
+
    if (files == NULL) return;	/* shouldn't happen */
 
    /* third mouse button cancels selection and reverts buffer to cwd name */
@@ -631,16 +682,18 @@ void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
 void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   Arg wargs[2];
    Window swin = xcWindow(w);
    Dimension swidth, sheight;
    int pstart, pheight, finscr;
-   short n = 0;
 
 #ifdef TCL_WRAPPER
    swidth = Tk_Width(w);
    sheight = Tk_Height(w);
 #else
+
+   Arg wargs[2];
+   short n = 0;
+
    XtnSetArg(XtNwidth, &swidth);
    XtnSetArg(XtNheight, &sheight);
    XtGetValues(w, wargs, n);
@@ -668,15 +721,16 @@ void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
 void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
-   Arg wargs[1];
    Dimension sheight;
    int phheight, finscr, flsave = flstart;
-   short n = 0;
    xcWidget filew = okaystruct->filew;
 
 #ifdef TCL_WRAPPER
    sheight = Tk_Height(w);
 #else
+   Arg wargs[1];
+   short n = 0;
+
    XtnSetArg(XtNheight, &sheight);
    XtGetValues(w, wargs, n);
 #endif
@@ -812,6 +866,8 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 
 #endif 	/* TCL_WRAPPER */
 
+#endif
+
 /*-------------------------------------------------------------------------*/
 /* Look for a directory name in a string and update cwdname accordingly	   */
 /*-------------------------------------------------------------------------*/
@@ -819,11 +875,13 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 int lookdirectory(char *lstring, int nchars)
 {
    int slen;
+   DIR *cwd = NULL;
 
    xc_tilde_expand(lstring, nchars);
    slen = strlen(lstring);
 
-   if (lstring[slen - 1] == '/' || (opendir(lstring) != NULL)) {
+   if (lstring[slen - 1] == '/' || ((cwd=opendir(lstring)) != NULL)) {
+      if (cwd) closedir(cwd);
       if (lstring[slen - 1] != '/') strcat(lstring, "/");
       cwdname = (char *)realloc(cwdname, (slen + 2) * sizeof(char));
       strcpy(cwdname, lstring);
