@@ -8,14 +8,18 @@
 #include <ctype.h>
 #include <string.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #endif
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes                                                          */
@@ -53,11 +57,13 @@ extern Tcl_Interp *xcinterp;
 FILE *findfontfile(char *fontname)
 {
    int i;
-   char tempname[100];
-   char *tmp_s = getenv((const char *)"XCIRCUIT_LIB_DIR");
+   char tempname[256];
    FILE *fd;
 
-   strcpy(_STR, fontname);
+   /* Add subdirectory "fonts".  We will try both with and	*/
+   /* without the subdirectory.					*/
+
+   sprintf(_STR, "fonts/%s", fontname);
 
    /* change string to all lowercase and remove dashes */
 
@@ -68,100 +74,69 @@ FILE *findfontfile(char *fontname)
 
    /* Fprintf(stdout, "Searching for font file \"%s\"\n", _STR); */
 
-   /* pick up file to load from _STR (defined from rcfile.c and files.c) */
-   /* try first in current directory, then in default library and subdir "fonts" */
+   /* Use the mechanism of "libopen" to find the encoding file	*/
+   /* in the search path					*/
 
-   if ((fd = fopen(_STR, "r")) == NULL) {
-      if (tmp_s != NULL) {
-         sprintf(tempname, "%s/%s", tmp_s, _STR);
-         fd = fopen(tempname, "r");
-         if (fd == NULL) {
-            sprintf(tempname, "%s/%s.xfe", tmp_s, _STR);
-            fd = fopen(tempname, "r");
-         }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s", tmp_s, _STR);
-            fd = fopen(tempname, "r");
-         }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s.xfe", tmp_s, _STR);
-            fd = fopen(tempname, "r");
-         }
+   fd = libopen(_STR + 6, FONTENCODING, NULL, 0);
+
+   if (fd == NULL) fd = libopen(_STR, FONTENCODING, NULL, 0);
+
+   /* Some other, probably futile, attempts (call findfontfile recursively) */
+
+   if (fd == NULL) {
+      char *dashptr;
+
+      /* If this font has a suffix, remove it and look for its root */
+      /* font on the supposition that this font is derived from the */
+      /* root font.						    */
+
+      strncpy(tempname, fontname, 99);
+      if ((dashptr = strrchr(tempname, '-')) != NULL) {
+	 *dashptr = '\0';
+	 if ((fd = findfontfile(tempname)) != NULL) return fd;
+
+	 /* And finally, because it's a common case, try adding	*/
+	 /* -Roman and trying again (but don't infinite loop!)	*/
+
+	 if (strcmp(dashptr + 1, "Roman")) {
+	    strcat(dashptr, "-Roman");
+	    if ((fd = findfontfile(tempname)) != NULL) return fd;
+	 }
       }
 
-      /* last resort:  hard-coded directory BUILTINS_DIR and subdir "fonts" */
+      Wprintf("No font encoding file found.");
+      if (fontcount > 0) {	/* Make font substitution */
+	 char *dchr, *psname = NULL;
+	 short fval;
 
-      if (fd == NULL) {
-         sprintf(tempname, "%s/%s", BUILTINS_DIR, _STR);
-         fd = fopen(tempname, "r");
-         if (fd == NULL) {
-            sprintf(tempname, "%s/%s.xfe", BUILTINS_DIR, _STR);
-            fd = fopen(tempname, "r");
+	 if ((dchr = strrchr(_STR, '.')) != NULL) *dchr = '\0';
+	 if ((fval = findhelvetica()) == fontcount) {
+	    /* This will cause some chaos. . . */
+	    Fprintf(stderr, "Error:  No fonts available!  Check library path?\n");
+	    exit(1);
 	 }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s", BUILTINS_DIR, _STR);
-            fd = fopen(tempname, "r");
-	 }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s.xfe", BUILTINS_DIR, _STR);
-            fd = fopen(tempname, "r");
-	 }
-         if (fd == NULL) {
-	    char *dashptr;
 
-	    /* If this font has a suffix, remove it and look for its root */
-	    /* font on the supposition that this font is derived from the */
-	    /* root font.						  */
-
-	    strncpy(tempname, fontname, 99);
-	    if ((dashptr = strrchr(tempname, '-')) != NULL) {
-	       *dashptr = '\0';
-	       if ((fd = findfontfile(tempname)) != NULL) return fd;
-
-	       /* And finally, because it's a common case, try adding	*/
-	       /* -Roman and trying again (but don't infinite loop!)	*/
-
-	       if (strcmp(dashptr + 1, "Roman")) {
-	          strcat(dashptr, "-Roman");
-	          if ((fd = findfontfile(tempname)) != NULL) return fd;
-	       }
-	    }
-
-            Wprintf("No font encoding file found.");
-	    if (fontcount > 0) {	/* Make font substitution */
-	       char *dchr, *psname = NULL;
-	       short fval;
-
-	       if ((dchr = strrchr(_STR, '.')) != NULL) *dchr = '\0';
-	       if ((fval = findhelvetica()) == fontcount) {
-		  /* This will cause some chaos. . . */
-		  Fprintf(stderr,
-			  "Error:  No fonts available!  Check library path?\n");
-		  exit(1);
-	       }
-	       psname = (char *)malloc((1 + strlen(fontname)) * sizeof(char));
-	       strcpy(psname, fontname);
-	       Wprintf("No encoding file found for font %s: substituting %s",
+	 psname = (char *)malloc((1 + strlen(fontname)) * sizeof(char));
+	 strcpy(psname, fontname);
+	 Wprintf("No encoding file found for font %s: substituting %s",
 			psname, fonts[fval].psname);
-               fonts = (fontinfo *)realloc(fonts, (fontcount + 1) * sizeof(fontinfo));
-               fonts[fontcount].psname = psname;
-               fonts[fontcount].family = psname;
-               fonts[fontcount].encoding = fonts[fval].encoding;
-               fonts[fontcount].flags = 0; 
-               fonts[fontcount].scale = 1.0;
-               fontcount++;
-	       makenewfontbutton();
-	    }
-	    else {
-	       Fprintf(stderr, "Error:  font encoding file missing for font \"%s\"\n",
-			fontname);
-	       Fprintf(stderr, "No fonts exist for a subsitution.  Make sure "
-			"fonts are installed or that\nenvironment variable "
-			"XCIRCUIT_LIB_DIR points to a directory of valid fonts.\n");
-	    }
-            return (FILE *)NULL;
-         }
+         fonts = (fontinfo *)realloc(fonts, (fontcount + 1) * sizeof(fontinfo));
+         fonts[fontcount].psname = psname;
+         fonts[fontcount].family = psname;
+         fonts[fontcount].encoding = fonts[fval].encoding;
+         fonts[fontcount].flags = 0; 
+         fonts[fontcount].scale = 1.0;
+         fontcount++;
+	 makenewfontbutton();
       }
+      else {
+	 Fprintf(stderr, "Error:  font encoding file missing for font \"%s\"\n",
+		fontname);
+	 Fprintf(stderr, "No fonts exist for a subsitution.  Make sure "
+		"fonts are installed or that\nenvironment variable "
+		"XCIRCUIT_LIB_DIR points to a directory of valid fonts.\n");
+      }
+      return (FILE *)NULL;
    }
    return fd;
 }
@@ -212,6 +187,7 @@ int loadfontfile(char *fontname)
          for (i = 0; i < fontcount; i++) {
             if (!strcmp(fonts[i].psname, tempname)) {
 	       /* Fprintf(stdout, "Font already loaded.\n"); */
+	       fclose(fd);
 	       return 0;
 	    }
 	 }
@@ -227,7 +203,8 @@ int loadfontfile(char *fontname)
 	 /* current version number for the file load.			*/
 
 	 version = PROG_VERSION;
-	 loadlibrary(FONTLIB);
+	 if (loadlibrary(FONTLIB) == FALSE) {
+	 };
 	 version = saveversion;
       }
 
@@ -316,7 +293,7 @@ int loadfontfile(char *fontname)
 	 if (!strcmp(tempname, "special") || !strcmp(tempname, "Special")) {
 	    flags |= 0x80;
 #ifdef TCL_WRAPPER
-	    Tcl_Eval(xcinterp, "xcircuit::newencodingbutton special");
+	    XcInternalTagCall(xcinterp, 3, "label", "encoding", "Special");
 #else
 	    makenewencodingbutton("Special", (char)1);
 #endif
@@ -325,14 +302,13 @@ int loadfontfile(char *fontname)
 	 /* ISO-LatinX encodings where X=1 to 6 */
 
 	 if (strstr(tempname, "ISO") != NULL) {
-	    char estr[38];
+	    char estr[12];
 	    for (i = 0; i < 6; i++) {
 	       if (strchr(tempname, '1' + (char)i) != NULL) {
 	          flags |= ((i + 2) << 7);
 #ifdef TCL_WRAPPER
-		  snprintf(estr, sizeof(estr),
-			"xcircuit::newencodingbutton ISOLatin%d", i + 1);
-		  Tcl_Eval(xcinterp, estr);
+		  snprintf(estr, sizeof(estr), "ISOLatin%d", i + 1);
+		  XcInternalTagCall(xcinterp, 3, "label", "encoding", estr);
 #else
 		  snprintf(estr, sizeof(estr), "ISO-Latin%d", i + 1);
 	          makenewencodingbutton(estr, (char)(i + 2));
@@ -403,5 +379,6 @@ int loadfontfile(char *fontname)
 	    makenewfontbutton();
       }
    }
+   fclose(fd);
    return 1;
 }
