@@ -7,8 +7,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef _MSC_VER
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -87,7 +89,7 @@ void loadlgf(int mode)
 
    /* Make sure that LGF object library has been loaded by loading it now. */
 
-   if (check_library(LGF_LIB) < 0) {
+   if (NameToLibrary(LGF_LIB) < 0) {
       int ilib;
       strcpy(_STR, LGF_LIB);
       ilib = createlibrary(FALSE);
@@ -232,7 +234,6 @@ void loadlgf(int mode)
 		  stringpart *strptr;
 
 		  NEW_LABEL(newlabel, topobject);
-	          topobject->parts++;
 
 		  labeldefaults(*newlabel, False, xmat(pvalx), ymat(pvaly));
 	          (*newlabel)->justify = TOP | NOTBOTTOM;
@@ -264,7 +265,6 @@ void loadlgf(int mode)
 	       /* Allocate wire */
 
 	       NEW_POLY(newwire, topobject);
-	       topobject->parts++;
 
 	       sscanf(lineptr, "%hd %hd %hd %hd", &pvalx, &pvaly, &pvalx2, &pvaly2);
 	       (*newwire)->number = 2;
@@ -315,7 +315,6 @@ void loadlgf(int mode)
 	       } while (*lineptr == '\n');
 
 	       NEW_POLY(newpoly, topobject);
-	       topobject->parts++;
 
 	       (*newpoly)->style = DASHED;
 	       (*newpoly)->color = DEFAULTCOLOR;
@@ -377,7 +376,6 @@ void loadlgf(int mode)
 	       if (j < xobjs.numlibs || k < xobjs.userlibs[xobjs.numlibs - 1].number) {
 
 		  NEW_OBJINST(newinst, topobject);
-      	          topobject->parts++;
 
 	          sscanf(lineptr, "%hd %hd %hd %*d %*d %*d %d", &pvalx, &pvaly,
 			&pvalx2, &hval); 
@@ -415,7 +413,6 @@ void loadlgf(int mode)
 			   stringpart *strptr;
 
 			   NEW_LABEL(newlabel, topobject);
-			   topobject->parts++;
 			   /* reconnect newinst if displaced by realloc() */
 			   newinst = (objinstptr *)(topobject->plist
 				+ topobject->parts - 2);
