@@ -49,13 +49,17 @@ keybinding *keylist = NULL;
 
 extern Display *dpy;
 extern char  _STR[150], _STR2[250];
-extern Boolean pressmode;
+extern int pressmode;
 extern XCWindowData *areawin;
 
-/*--------------------------------------------------------------*/
-/* Key modifiers (convenience definitions)			*/
-/* Use Mod2Mask for a "hold" ("press") definition.		*/
-/*--------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Key modifiers (convenience definitions)				*/
+/* Use Mod5Mask for a "hold" ("press") definition.			*/
+/* Why Mod5Mask?  Mod2Mask is used by fvwm2 for some obscure purpose,	*/
+/* so I went to the other end (Mod5Mask is just below Button1Mask).	*/
+/* Thanks to John Heil for reporting the problem and confirming the	*/
+/* Mod5Mask solution.							*/
+/*----------------------------------------------------------------------*/
 
 #define ALT (Mod1Mask << 16)
 #define CTRL (ControlMask << 16)
@@ -66,7 +70,7 @@ extern XCWindowData *areawin;
 #define BUTTON3 (Button3Mask << 16)
 #define BUTTON4 (Button4Mask << 16)
 #define BUTTON5 (Button5Mask << 16)
-#define HOLD (Mod2Mask << 16)
+#define HOLD (Mod4Mask << 16)
 
 #define ALL_WINDOWS (xcWidget)NULL
 
@@ -84,7 +88,7 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Text Return", "Text Delete", "Text Right", "Text Left",
    "Text Up", "Text Down", "Text Split", 
    "Text Home", "Text End", "Linebreak", "Parameter",
-   "Parameterize Point", "Break at Point", "Delete Point", "Insert Point",
+   "Parameterize Point", "Change Style", "Delete Point", "Insert Point",
    "Append Point", "Next Point", "Attach", "Next Library", "Library Directory",
    "Library Move", "Library Copy", "Library Edit", "Library Delete",
    "Library Duplicate", "Library Hide", "Library Virtual Copy",
@@ -97,10 +101,10 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Undo", "Redo", "Select Save", "Unselect", "Dashed", "Dotted",
    "Solid", "Prompt", "Dot", "Wire", "Cancel", "Nothing", "Exit",
    "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label", "Graphic",
-   "Rescale", "Select Box", "Connectivity", "Continue Copy",
-   "Continue Element", "Finish Copy", "Finish Element", "Finish",
-   "Cancel Last", "Sim", "SPICE", "PCB", "SPICE Flat", "Color",
-   "Text Delete Param", "Scale", "Change Style",
+   "Select Box", "Connectivity", "Continue Element", "Finish Element",
+   "Continue Copy", "Finish Copy", "Finish", "Cancel Last", "Sim",
+   "SPICE", "PCB", "SPICE Flat", "Rescale", "Reorder", "Color",
+   "Text Delete Param",
    NULL			/* sentinel */
 };
 
@@ -131,10 +135,12 @@ int firstbinding(xcWidget window, int function)
    int keywstate = -1;
 
    for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
-      if (ksearch->window == window)
-	 return ksearch->keywstate;
-      else if (ksearch->window == ALL_WINDOWS)
-	 keywstate = ksearch->keywstate;
+      if (ksearch->function == function) {
+         if (ksearch->window == window)
+	    return ksearch->keywstate;
+         else if (ksearch->window == ALL_WINDOWS)
+	    keywstate = ksearch->keywstate;
+      }
    }
    return keywstate;
 }
@@ -428,7 +434,7 @@ char *key_to_string(int keywstate)
    kptr = (char *)malloc(32);
    kptr[0] = '\0';
    if (kmod & Mod1Mask) strcat(kptr, "Alt_");
-   if (kmod & Mod2Mask) strcat(kptr, "Hold_");
+   if (kmod & Mod4Mask) strcat(kptr, "Hold_");
    if (kmod & ControlMask) strcat(kptr, "Control_");
    if (kmod & LockMask) strcat(kptr, "Capslock_");
    if (kmod & ShiftMask) strcat(kptr, "Shift_");
@@ -469,13 +475,6 @@ void printeditbindings()
 
    _STR2[0] = '\0';
 
-   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Break));
-   strcat(_STR2, tstr);
-   strcat(_STR2, "=");
-   strcat(_STR2, func_to_string(XCF_Edit_Break));
-   strcat(_STR2, ", ");
-   free(tstr);
-
    tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Delete));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
@@ -671,11 +670,12 @@ void default_keybindings()
    add_binding(ALL_WINDOWS, ALT | XK_Return, XCF_Linebreak);
    add_binding(ALL_WINDOWS, ALT | XK_h, XCF_Halfspace);
    add_binding(ALL_WINDOWS, ALT | XK_q, XCF_Quarterspace);
+#ifndef TCL_WRAPPER
+   add_binding(ALL_WINDOWS, ALT | XK_p, XCF_Parameter);
+#endif
    add_binding(ALL_WINDOWS, XK_backslash, XCF_Special);
    add_binding(ALL_WINDOWS, ALT | XK_c, XCF_Special);
-   add_binding(ALL_WINDOWS, ALT | XK_p, XCF_Parameter);
    add_binding(ALL_WINDOWS, XK_p, XCF_Edit_Param);
-   add_binding(ALL_WINDOWS, XK_x, XCF_Edit_Break);
    add_binding(ALL_WINDOWS, XK_d, XCF_Edit_Delete);
    add_binding(ALL_WINDOWS, XK_Delete, XCF_Edit_Delete);
    add_binding(ALL_WINDOWS, XK_i, XCF_Edit_Insert);
@@ -688,11 +688,13 @@ void default_keybindings()
    add_binding(ALL_WINDOWS, XK_L, XCF_Library_Directory);
    add_binding(ALL_WINDOWS, XK_c, XCF_Library_Copy);
    add_binding(ALL_WINDOWS, XK_E, XCF_Library_Edit);
+   add_binding(ALL_WINDOWS, XK_e, XCF_Library_Edit);
    add_binding(ALL_WINDOWS, XK_D, XCF_Library_Delete);
    add_binding(ALL_WINDOWS, XK_C, XCF_Library_Duplicate);
    add_binding(ALL_WINDOWS, XK_H, XCF_Library_Hide);
    add_binding(ALL_WINDOWS, XK_V, XCF_Library_Virtual);
    add_binding(ALL_WINDOWS, XK_M, XCF_Library_Move);
+   add_binding(ALL_WINDOWS, XK_m, XCF_Library_Move);
    add_binding(ALL_WINDOWS, XK_P, XCF_Page_Directory);
    add_binding(ALL_WINDOWS, XK_less, XCF_Library_Pop);
    add_binding(ALL_WINDOWS, HOLD | BUTTON1, XCF_Library_Pop);
@@ -778,7 +780,7 @@ void default_keybindings()
    /* 2-button vs. 3-button mouse bindings (set with -2	*/
    /* commandline option; 3-button bindings default)	*/
 
-   if (pressmode) {
+   if (pressmode == 1) {
       add_binding(ALL_WINDOWS, BUTTON3, XCF_Text_Return);
       add_binding(ALL_WINDOWS, BUTTON3, XCF_Select);
       add_binding(ALL_WINDOWS, HOLD | BUTTON3, XCF_SelectBox);
