@@ -932,8 +932,10 @@ extern int findcurfont(int, stringpart *, objinstptr);
 extern stringpart *findtextinstring(char *, int *, stringpart *, objinstptr);
 extern stringpart *findstringpart(int, int *, stringpart *, objinstptr);
 extern void charprint(char *, stringpart *, int);
+extern void charprinttex(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
+extern char *textprinttex(stringpart *, objinstptr);
 extern char *textprintsubnet(stringpart *, objinstptr, int);
 extern char *textprintnet(char *, char *, Genericlist *);
 extern int textcomp(stringpart *, char *, objinstptr);
