@@ -13,15 +13,21 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
+#include <unistd.h>     /* for usleep() */
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #endif
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes                                                       */
@@ -121,7 +127,7 @@ short execcommand(short pflags, char *cmdptr)
       if (!strncmp(argptr, "default", 7)) {
 	 argptr += 7;
 	 while (isspace(*argptr) && (*argptr != '\0')) argptr++;
-	 if (*argptr == '\0') return;	/* bad syntax */
+	 if (*argptr == '\0') return flags;	/* bad syntax */
       }
 
       sscanf(argptr, "%49s", value);
@@ -146,7 +152,7 @@ short execcommand(short pflags, char *cmdptr)
       int savepage = areawin->page;
       if (sscanf(argptr, "%249s %d", _STR, &pageno) >= 1) {
 	 if (pageno != savepage) newpage(pageno);
-	 startloadfile();
+	 startloadfile(-1);
 	 if (pageno != savepage) newpage(savepage);
       }
       else
@@ -283,7 +289,7 @@ short execcommand(short pflags, char *cmdptr)
       if (!strncmp(argptr, "default", 7)) {
 	 argptr += 7;
 	 while (isspace(*argptr) && (*argptr != '\0')) argptr++;
-	 if (*argptr == '\0') return;	/* bad syntax */
+	 if (*argptr == '\0') return flags;	/* bad syntax */
       }
 
       if (!strncmp(argptr, "font", 4)) {
@@ -397,6 +403,7 @@ short readcommand(short mode, FILE *fd)
    }
 
    free(temp);
+   fclose(fd);
    return flags;
 }
 
@@ -408,13 +415,13 @@ short readcommand(short mode, FILE *fd)
 void defaultscript()
 {
    FILE *fd;
-   char *tmp_s = getenv((const char *)"XCIRCUIT_LIB_DIR");
+   char *tmp_s = getenv((const char *)"XCIRCUIT_SRC_DIR");
 
-   if (!tmp_s) tmp_s = BUILTINS_DIR;
+   if (!tmp_s) tmp_s = SCRIPTS_DIR;
    sprintf(_STR2, "%s/%s", tmp_s, STARTUP_FILE);
 
    if ((fd = fopen(_STR2, "r")) == NULL) {
-      sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
+      sprintf(_STR2, "%s/%s", SCRIPTS_DIR, STARTUP_FILE);
       if ((fd = fopen(_STR2, "r")) == NULL) {
          Wprintf("Failed to open startup script \"%s\"\n", STARTUP_FILE);
 	 return;
