@@ -7,21 +7,33 @@
 /* We use OpenGL functions to read and scale images if OpenGL is enabled */
 #ifndef OPENGL
 
-#undef GS_DEBUG
-/* #define GS_DEBUG */
+/* #undef GS_DEBUG */
+#define GS_DEBUG
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef XC_WIN32
+#include <termios.h>    /* termios do not exist on windows */
 #include <unistd.h>
+#endif
 #include <time.h>
+#ifndef XC_WIN32
 #include <sys/wait.h>   /* for waitpid() */
+#endif
 #include <sys/types.h>
 #include <signal.h>
 #include <fcntl.h>
+
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xatom.h>
+#else
+#ifdef TCL_WRAPPER
+#include <X11/Xatom.h>
+#endif
+#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -45,9 +57,8 @@
 
 extern char _STR2[250], _STR[150];
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Display *dpy;
-extern short eventmode;
 extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
@@ -73,7 +84,11 @@ int gs_state;		/* Track state of the renderer */
 /*------------------------------------------------------*/
 
 Atom  gv, gvc, gvpage, gvnext, gvdone;
+#ifndef _MSC_VER
 pid_t gsproc = -1;	/* ghostscript process 			*/
+#else
+HANDLE gsproc = INVALID_HANDLE_VALUE;
+#endif
 int   fgs[2];		/* stdin pipe pair for ghostscript	*/
 Window mwin = 0;	/* "false" window hack to get gs	*/
 			/* process to capture ClientMessage	*/
@@ -90,13 +105,13 @@ void ghostinit_local()
 {
    sprintf(_STR, "%ld %d %d %d %d %d %g %g %d %d %d %d",
 		0L, 0, 0, 0,
-		areastruct.width * 75 / 72,
-		areastruct.height * 75 / 72,
+		areawin->width * 75 / 72,
+		areawin->height * 75 / 72,
 		75.0, 75.0, 0, 0, 0, 0);
-   XChangeProperty(dpy, areastruct.areawin, gv, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(dpy, areawin->window, gv, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    sprintf(_STR, "%s %d %d", "Color", (int)FOREGROUND, (int)BACKGROUND);
-   XChangeProperty(dpy, areastruct.areawin, gvc, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(dpy, areawin->window, gvc, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    XSync(dpy, False);
 
@@ -120,13 +135,13 @@ Boolean render_client(XEvent *eventPtr)
 #endif
       mwin = eventPtr->xclient.data.l[0];
       Wprintf("Background finished.");
-      XDefineCursor(dpy, areastruct.areawin, DEFAULTCURSOR);
+      XDefineCursor(dpy, areawin->window, DEFAULTCURSOR);
 
       /* Mark this as the most recently rendered background, so we don't	*/
       /* have to render more than necessary.					*/
-      areastruct.lastbackground = xobjs.pagelist[areastruct.page]->background.name;
+      areawin->lastbackground = xobjs.pagelist[areawin->page]->background.name;
       gs_state = GS_READY;
-      drawarea(areastruct.area, NULL, NULL);
+      drawarea(areawin->area, NULL, NULL);
    }
    else if (eventPtr->xclient.message_type == gvdone) {
 #ifdef GS_DEBUG
@@ -187,7 +202,7 @@ void send_client(Atom msg)
 
    event.xclient.type = ClientMessage;
    event.xclient.display = dpy;
-   event.xclient.window = areastruct.areawin;
+   event.xclient.window = areawin->window;
    event.xclient.message_type = msg;
    event.xclient.format = 32;
    event.xclient.data.l[0] = mwin;
@@ -226,6 +241,10 @@ void ask_for_next()
 void start_gs()
 {
    int std_out[2], std_err[2], ret;
+#ifdef XC_WIN32
+   unsigned int pipe_size = 8196; /* Windows need a pipe size argument */
+   int pipe_mode = _O_BINARY;     /* Windows need a pipe mode argument */
+#endif
 #ifdef HAVE_PUTENV
    static char env_str1[128], env_str2[64];
 #endif
@@ -233,32 +252,42 @@ void start_gs()
 #ifdef TCL_WRAPPER
    if (bbuf != (Pixmap)NULL) Tk_FreePixmap(dpy, bbuf);
 #ifdef DOUBLEBUFFER
-   bbuf = Tk_GetPixmap(dpy, dbuf, areastruct.width, areastruct.height,
-		Tk_Depth(areastruct.area));
+   bbuf = Tk_GetPixmap(dpy, dbuf, areawin->width, areawin->height,
+		Tk_Depth(areawin->area));
 #else
-   bbuf = Tk_GetPixmap(dpy, (Drawable)areastruct.areawin,
-		areastruct.width, areastruct.height,
-		Tk_Depth(areastruct.area));
+   bbuf = Tk_GetPixmap(dpy, (Drawable)areawin->window,
+		areawin->width, areawin->height,
+		Tk_Depth(areawin->area));
 #endif
 
 #else	/* !TCL_WRAPPER */
    if (bbuf != (Pixmap)NULL) XFreePixmap(dpy, bbuf);
 #ifdef DOUBLEBUFFER
-   bbuf = XCreatePixmap(dpy, dbuf, areastruct.width, areastruct.height,
-		DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+   bbuf = XCreatePixmap(dpy, dbuf, areawin->width, areawin->height,
+		DefaultDepthOfScreen(xcScreen(areawin->area))); 
 #else
-   bbuf = XCreatePixmap(dpy, areastruct.areawin,
-		areastruct.width, areastruct.height,
-		DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+   bbuf = XCreatePixmap(dpy, areawin->window,
+		areawin->width, areawin->height,
+		DefaultDepthOfScreen(xcScreen(areawin->area))); 
 #endif	/* DOUBLEBUFFER */
 #endif  /* TCL_WRAPPER */
 
    XSync(dpy, False);
 
+#ifndef XC_WIN32
    ret = pipe(fgs);
    ret = pipe(std_out);
+#else
+   ret = _pipe(fgs, pipe_size, pipe_mode);
+   ret = _pipe(std_out, pipe_size, pipe_mode);
+#endif
+
 #ifndef GS_DEBUG
+#ifndef XC_WIN32
    ret = pipe(std_err);
+#else
+   ret = _pipe(srd_err, pipe_size, pipe_mode);
+#endif /* XC_WIN32 */
 #endif
 
    /* We need a complicated pipe here, with input going from xcircuit	*/
@@ -266,6 +295,58 @@ void start_gs()
    /* the background file to gs for rendering.				*/
    /* Here is not the place to do it.  Set up gs to take stdin as input.*/
 
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) {
+	 STARTUPINFO st_info;
+	 PROCESS_INFORMATION pr_info;
+	 char cmd[4096];
+
+#ifdef HAVE_PUTENV
+   	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
+	 putenv(env_str1);
+#ifdef DOUBLEBUFFER
+   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areawin->window, (long)bbuf);
+	 putenv(env_str2);
+#else
+	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areawin->window);
+	 putenv(env_str2);
+#endif
+#else
+	 setenv("DISPLAY", XDisplayString(dpy), True);
+#ifdef DOUBLEBUFFER
+	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
+	 setenv("GHOSTVIEW", _STR, True);
+#else
+ 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
+	 setenv("GHOSTVIEW", _STR, True);
+#endif
+#endif
+
+	 SetHandleInformation((HANDLE)_get_osfhandle(fgs[1]), HANDLE_FLAG_INHERIT, 0);
+	 SetHandleInformation((HANDLE)_get_osfhandle(std_out[0]), HANDLE_FLAG_INHERIT, 0);
+#ifndef GS_DEBUG
+	 SetHandleInformation((HANDLE)_get_osfhandle(std_err[0]), HANDLE_FLAG_INHERIT, 0);
+#endif
+
+	 ZeroMemory(&st_info, sizeof(STARTUPINFO));
+	 ZeroMemory(&pr_info, sizeof(PROCESS_INFORMATION));
+	 st_info.cb = sizeof(STARTUPINFO);
+	 st_info.dwFlags = STARTF_USESTDHANDLES;
+	 st_info.hStdOutput = (HANDLE)_get_osfhandle(std_out[1]);
+	 st_info.hStdInput = (HANDLE)_get_osfhandle(fgs[0]);
+#ifndef GS_DEBUG
+	 st_info.hStdError = (HANDLE)_get_osfhandle(std_err[1]);
+#endif
+
+	 snprintf(cmd, 4095, "%s -dNOPAUSE -", GS_EXEC);
+	 if (CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL, &st_info, &pr_info) == 0) {
+		 Wprintf("Error: ghostscript not running");
+		 return;
+	 }
+	 CloseHandle(pr_info.hThread);
+	 gsproc = pr_info.hProcess;
+   }
+#else
    if (gsproc < 0) {  /* Ghostscript is not running yet */
       gsproc = vfork();
       if (gsproc == 0) {		/* child process (gs) */
@@ -289,24 +370,24 @@ void start_gs()
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
 #ifdef DOUBLEBUFFER
-   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areastruct.areawin, (long)bbuf);
+   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areawin->window, (long)bbuf);
 	 putenv(env_str2);
 #else
-	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areastruct.areawin);
+	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areawin->window);
 	 putenv(env_str2);
 #endif
 #else
 	 setenv("DISPLAY", XDisplayString(dpy), True);
 #ifdef DOUBLEBUFFER
-	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
+	 sprintf(_STR, "%ld %ld", (long)areawin->window, (long)bbuf);
 	 setenv("GHOSTVIEW", _STR, True);
 #else
- 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
+ 	 sprintf(_STR, "%ld", (long)areawin->window);
 	 setenv("GHOSTVIEW", _STR, True);
 #endif
 #endif
 	 Flush(stderr);
-         execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", NULL);
+         execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", (char *)NULL);
 	 gsproc = -1;
 	 fprintf(stderr, "Exec of gs failed\n");
 	 return;
@@ -316,6 +397,7 @@ void start_gs()
 	 return;			/* error condition */
       }
    }
+#endif
 }
 
 /*--------------------------------------------------------*/
@@ -329,7 +411,7 @@ void parse_bg(FILE *fi, FILE *fbg) {
    char line_in[256];
    float psscale;
 
-   psscale = getpsscale(xobjs.pagelist[areastruct.page]->outscale, areastruct.page);
+   psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale, areawin->page);
 
    for(;;) {
       if (fgets(line_in, 255, fi) == NULL) {
@@ -357,10 +439,10 @@ void parse_bg(FILE *fi, FILE *fbg) {
 			llx, lly, urx, ury);
 #endif
 	
-	       xobjs.pagelist[areastruct.page]->background.bbox.lowerleft.x = llx;
-	       xobjs.pagelist[areastruct.page]->background.bbox.lowerleft.y = lly;
-	       xobjs.pagelist[areastruct.page]->background.bbox.width = (urx - llx);
-	       xobjs.pagelist[areastruct.page]->background.bbox.height = (ury - lly);
+	       xobjs.pagelist[areawin->page]->background.bbox.lowerleft.x = llx;
+	       xobjs.pagelist[areawin->page]->background.bbox.lowerleft.y = lly;
+	       xobjs.pagelist[areawin->page]->background.bbox.width = (urx - llx);
+	       xobjs.pagelist[areawin->page]->background.bbox.height = (ury - lly);
 	       if (fbg == (FILE *)NULL) break;
 	    }
 	 }
@@ -378,7 +460,7 @@ void bg_get_bbox()
    FILE *fi;
    char *fname;
 
-   fname = xobjs.pagelist[areastruct.page]->background.name;
+   fname = xobjs.pagelist[areawin->page]->background.name;
    if ((fi = fopen(fname, "r")) == NULL) {
       fprintf(stderr, "Failure to open background file to get bounding box info\n");
       return;
@@ -430,7 +512,11 @@ void readbackground(FILE *fi)
    /* "@" denotes a temporary file */
    sprintf(file_in, "@%s/XXXXXX", xobjs.tempdir);
 
+#ifdef _MSC_VER
+   tfd = mktemp(file_in + 1);
+#else
    tfd = mkstemp(file_in + 1);
+#endif
    if (tfd == -1) fprintf(stderr, "Error generating temporary filename\n");
    else {
       if ((fbg = fdopen(tfd, "w")) == NULL) {
@@ -491,9 +577,9 @@ void register_bg(char *gsfile)
    else
       reset_gs();
 
-   xobjs.pagelist[areastruct.page]->background.name =
+   xobjs.pagelist[areawin->page]->background.name =
 		(char *) malloc(strlen(gsfile) + 1);
-   strcpy(xobjs.pagelist[areastruct.page]->background.name, gsfile);
+   strcpy(xobjs.pagelist[areawin->page]->background.name, gsfile);
 }
 
 /*------------------------------------------------------*/
@@ -509,7 +595,7 @@ void loadbackground()
    register_bg(_STR2);
    bg_get_bbox();
    updatepagebounds(topobject);
-   zoomview(areastruct.area, NULL, NULL);
+   zoomview(areawin->area, NULL, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -518,7 +604,12 @@ void loadbackground()
 
 void send_to_gs(char *text)
 {
+#ifndef XC_WIN32
    write(fgs[1], text, strlen(text));
+   tcflush(fgs[1], TCOFLUSH);
+#else
+   _write(fgs[1], text, (unsigned int)strlen(text));
+#endif
 #ifdef GS_DEBUG
    fprintf(stdout, "writing: %s", text);
 #endif
@@ -537,29 +628,29 @@ int renderbackground()
 
    if (gsproc < 0) return -1;
 
-   defscale = (xobjs.pagelist[areastruct.page]->coordstyle == CM) ?
+   defscale = (xobjs.pagelist[areawin->page]->coordstyle == CM) ?
 	CMSCALE : INCHSCALE;
 
-   psnorm = (*areastruct.vscale) * (1.0 / defscale) * devres;
+   psnorm = areawin->vscale * (1.0 / defscale) * devres;
 
-   psxpos = (float)(-areastruct.pcorner->x) * (*areastruct.vscale) * devres;
-   psypos = (float)(-areastruct.pcorner->y) * (*areastruct.vscale) * devres
-		+ ((float)areastruct.height / 12.0);
+   psxpos = (float)(-areawin->pcorner.x) * areawin->vscale * devres;
+   psypos = (float)(-areawin->pcorner.y) * areawin->vscale * devres
+		+ ((float)areawin->height / 12.0);
 
    /* Conditions for re-rendering:  Must have a background specified */
    /* and must be on the page, not a library or other object.	     */
 
-   if (xobjs.pagelist[areastruct.page]->background.name == (char *)NULL)
+   if (xobjs.pagelist[areawin->page]->background.name == (char *)NULL)
       return -1;
-   else if (areastruct.lastbackground
-		== xobjs.pagelist[areastruct.page]->background.name) {
+   else if (areawin->lastbackground
+		== xobjs.pagelist[areawin->page]->background.name) {
       return 0;
    }
 
    if (is_page(topobject) == -1)
       return -1;
 
-   bgfile = xobjs.pagelist[areastruct.page]->background.name;
+   bgfile = xobjs.pagelist[areawin->page]->background.name;
    if (*bgfile == '@') bgfile++;
 
    /* Ask ghostscript to produce the next page */
@@ -568,7 +659,7 @@ int renderbackground()
    /* Set the last background name to NULL; this will get the 	*/
    /* value when the rendering is done.				*/
 
-   areastruct.lastbackground = NULL;
+   areawin->lastbackground = NULL;
 
    /* write scale and position to ghostscript 		*/
    /* and tell ghostscript to run the requested file	*/
@@ -589,7 +680,7 @@ int renderbackground()
    fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
 #endif
    Wprintf("Rendering background image.");
-   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
+   XDefineCursor(dpy, areawin->window, WAITFOR);
 
    /* The page will be refreshed when we receive confirmation	*/
    /* from ghostscript that the buffer has been rendered.	*/
@@ -612,11 +703,11 @@ int copybackground()
       return -1;
 
 #ifdef DOUBLEBUFFER
-   XCopyArea(dpy, bbuf, dbuf, areastruct.gc, 0, 0,
-             areastruct.width, areastruct.height, 0, 0);
+   XCopyArea(dpy, bbuf, dbuf, areawin->gc, 0, 0,
+             areawin->width, areawin->height, 0, 0);
 #else
-   XCopyArea(dpy, bbuf, areastruct.areawin, areastruct.gc, 0, 0,
-             areastruct.width, areastruct.height, 0, 0);
+   XCopyArea(dpy, bbuf, areawin->window, areawin->gc, 0, 0,
+             areawin->width, areawin->height, 0, 0);
 #endif
 
    return 0;
@@ -628,19 +719,31 @@ int copybackground()
 
 int exit_gs()
 {
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) return -1;	/* gs not running */
+#else
    if (gsproc < 0) return -1;	/* gs not running */
+#endif
 
 #ifdef GS_DEBUG
    fprintf(stdout, "Waiting for gs to exit\n");
 #endif
+#ifndef _MSC_VER
    kill(gsproc, SIGKILL);
    waitpid(gsproc, NULL, 0);      
+#else
+   TerminateProcess(gsproc, -1);
+#endif
 #ifdef GS_DEBUG
    fprintf(stdout, "gs has exited\n");
 #endif
 
    mwin = 0;
+#ifdef _MSC_VER
+   gsproc = INVALID_HANDLE_VALUE;
+#else
    gsproc = -1;
+#endif
    gs_state = GS_INIT;
 
    return 0;
@@ -652,7 +755,11 @@ int exit_gs()
 
 int reset_gs()
 {
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) return -1;
+#else
    if (gsproc < 0) return -1;
+#endif
 
    exit_gs();
    ghostinit_local();
