@@ -45,8 +45,12 @@ extern Display	*dpy;
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
 extern XCWindowData *areawin;
+extern Globaldata xobjs;
 extern char _STR[150];
-extern short textpos, textend;
+
+#ifdef TCL_WRAPPER 
+extern Tcl_Interp *xcinterp;
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Prevent a list of elements from being selected.			*/
@@ -98,17 +102,24 @@ void selectfilter(xcWidget w, pointertype value, caddr_t calldata)
 #endif
 }
 
-/*----------------------------------------------------------------------------*/
-/* Look at select stack to see if there are any selects; call select, if not. */
-/*----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Look at select stack to see if there are any selects; call select,	*/
+/* if not.  If draw_selected is True, then the selected items are drawn	*/
+/* in the select color.	 Otherwise, they are not redrawn.		*/
+/*----------------------------------------------------------------------*/
 
-Boolean checkselect(short value)
+Boolean checkselect_draw(short value, Boolean draw_selected)
 {
-   short *check;
+   short *check, savemode;
  
    value &= areawin->filter;	/* apply the selection filter */
 
-   if (areawin->selects == 0) select_element(value);
+   if (areawin->selects == 0) {
+      savemode = eventmode;
+      if (!draw_selected) eventmode = PENDING_MODE;
+      select_element(value);
+      eventmode = savemode;
+   }
    if (areawin->selects == 0) return False;
    for (check = areawin->selectlist; check < areawin->selectlist +
 	areawin->selects; check++)
@@ -117,6 +128,15 @@ Boolean checkselect(short value)
    else return True;
 }
 
+/*----------------------------------------------------------------------------*/
+/* Look at select stack to see if there are any selects; call select, if not. */
+/*----------------------------------------------------------------------------*/
+
+Boolean checkselect(short value)
+{
+    return checkselect_draw(value, False);
+}
+
 /*--------------------------------------------------------------*/
 /* Select list numbering revision when an element is deleted	*/
 /* from an object. 						*/
@@ -138,29 +158,35 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 {
    genericptr elementptr = *(curobj->plist + instance);
 
+   // Note:  Setting areawin->clipped to -1 prevents the clipmask from being
+   // applied to elements, 
+   areawin->clipped = -1;
+
    switch (ELEMENTTYPE(*(curobj->plist + instance))) {
       case ARC:
-         UDrawArc((arcptr)elementptr);
+         UDrawArc((arcptr)elementptr, xobjs.pagelist[areawin->page]->wirewidth);
 	 break;
       case POLYGON:
-         UDrawPolygon((polyptr)elementptr);
+         UDrawPolygon((polyptr)elementptr, xobjs.pagelist[areawin->page]->wirewidth);
 	 break;
       case SPLINE:
-	 UDrawSpline((splineptr)elementptr);
+	 UDrawSpline((splineptr)elementptr, xobjs.pagelist[areawin->page]->wirewidth);
 	 break;
       case PATH:
-	 UDrawPath((pathptr)elementptr);
+	 UDrawPath((pathptr)elementptr, xobjs.pagelist[areawin->page]->wirewidth);
 	 break;
       case LABEL:
          UDrawString((labelptr)elementptr, mode, curinst);
 	 break;
       case OBJINST:
-         UDrawObject((objinstptr)elementptr, SINGLE, mode, NULL);
+         UDrawObject((objinstptr)elementptr, SINGLE, mode,
+			xobjs.pagelist[areawin->page]->wirewidth, NULL);
          break;
       case GRAPHIC:
          UDrawGraphic((graphicptr)elementptr);
 	 break;
    }
+   areawin->clipped = 0;
 }
 
 /*-------------------------------------------------*/
@@ -169,17 +195,25 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 
 void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 {
-   SetFunction(dpy, areawin->gc, GXcopy);
-   SetForeground(dpy, areawin->gc, BACKGROUND);
+   /* Don't draw selection color when selecting for edit */
+   if (eventmode == PENDING_MODE) return;
+
+   if (*newselect >= curobj->parts) return;	// Safety check
+
+   XcSetFunction(GXcopy);
+   XcSetForeground(BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
    indicateparams(*(curobj->plist + *newselect));
 
-   SetFunction(dpy, areawin->gc, GXxor);
-   SetForeground(dpy, areawin->gc, SELECTCOLOR ^ BACKGROUND);
+   XcSetFunction(GXxor);
+   XcSetForeground(SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
+
+   SetForeground(dpy, areawin->gc, areawin->gccolor);
+   SetFunction(dpy, areawin->gc, areawin->gctype);
 }
 
 /*---------------------------------------------------*/
@@ -370,6 +404,387 @@ Boolean pathselect(genericptr *curgen, short class, float range)
    return selected;
 }
 
+/*------------------------------------------------------*/
+/* Check to see if any selection has registered cycles	*/
+/*------------------------------------------------------*/
+
+Boolean  checkforcycles(short *selectlist, int selects)
+{
+   genericptr pgen;
+   pointselect *cycptr;
+   short *ssel;
+
+   for (ssel = selectlist; ssel < selectlist + selects; ssel++) {
+      pgen = SELTOGENERIC(ssel);
+      switch(pgen->type) {
+         case POLYGON:
+	    cycptr = ((polyptr)pgen)->cycle;
+	    break;
+         case ARC:
+	    cycptr = ((arcptr)pgen)->cycle;
+	    break;
+         case SPLINE:
+	    cycptr = ((splineptr)pgen)->cycle;
+	    break;
+         case LABEL:
+	    cycptr = ((labelptr)pgen)->cycle;
+	    break;
+      }
+      if (cycptr != NULL)
+	 if (cycptr->number != -1)
+	    return True;
+   }
+   return False;
+}
+
+/*--------------------------------------------------------------*/
+/* Copy a cycle selection list from one element to another	*/
+/*--------------------------------------------------------------*/
+
+void copycycles(pointselect **new, pointselect **old)
+{
+   pointselect *pptr;
+   short cycles = 0;
+
+   if (*old == NULL) {
+      *new = NULL;
+      return;
+   }
+
+   for (pptr = *old; !(pptr->flags & LASTENTRY); pptr++, cycles++);
+   cycles += 2;
+   *new = (pointselect *)malloc(cycles * sizeof(pointselect));
+   memcpy(*new, *old, cycles * sizeof(pointselect));
+}
+
+/*--------------------------------------------------------------*/
+/* Create a selection record of selected points in an element.	*/
+/* If a record already exists, and "cycle" is not already in	*/
+/* the list, add it.						*/
+/* "flags" may be set to EDITX or EDITY.  If "flags" is zero,	*/
+/* then flags = EDITX | EDITY is assumed.			*/
+/*--------------------------------------------------------------*/
+
+pointselect *addcycle(genericptr *pgen, short cycle, u_char flags)
+{
+   polyptr ppoly;
+   arcptr parc;
+   splineptr pspline;
+   labelptr plabel;
+   pointselect *pptr, **cycptr;
+   short cycles = 0;
+
+   switch((*pgen)->type) {
+      case POLYGON:
+	 ppoly = TOPOLY(pgen);
+	 cycptr = &ppoly->cycle;
+	 break;
+      case ARC:
+	 parc = TOARC(pgen);
+	 cycptr = &parc->cycle;
+	 break;
+      case SPLINE:
+	 pspline = TOSPLINE(pgen);
+	 cycptr = &pspline->cycle;
+	 break;
+      case LABEL:
+         plabel = TOLABEL(pgen);
+	 cycptr = &plabel->cycle;
+	 break;
+   }
+
+   switch((*pgen)->type) {
+      case POLYGON:
+      case ARC:
+      case SPLINE:
+      case LABEL:		// To-do:  Handle labels separately
+
+	 if (*cycptr == NULL) {
+	    *cycptr = (pointselect *)malloc(sizeof(pointselect));
+	    pptr = *cycptr;
+	    pptr->number = cycle;
+	    pptr->flags = (flags == 0) ? EDITX | EDITY : flags;
+	    pptr->flags |= LASTENTRY;
+	 }
+	 else {
+	    for (pptr = *cycptr; !(pptr->flags & LASTENTRY); pptr++, cycles++) {
+	       if (pptr->number == cycle)
+		  break;
+	       pptr->flags &= ~LASTENTRY;
+	    }
+	    if (pptr->number != cycle) {
+	       pptr->flags &= ~LASTENTRY;
+	       *cycptr = (pointselect *)realloc(*cycptr,
+			(cycles + 2) * sizeof(pointselect));
+	       pptr = *cycptr + cycles + 1;
+	       pptr->number = cycle;
+	       pptr->flags = (flags == 0) ? EDITX | EDITY : flags;
+	       pptr->flags |= LASTENTRY;
+	    }
+	    else {
+	       pptr->flags |= (flags == 0) ? EDITX | EDITY : flags;
+	    }
+	 }
+         break;
+   }
+   return pptr;
+}
+
+/*--------------------------------------------------------------*/
+/* If we edit the position of a control point, and the global	*/
+/* pathedit mode is set to TANGENTS, then we track the angle of	*/
+/* the adjoining curve, if there is one, by settings its cycle	*/
+/* flags to ANTIXY.						*/
+/*--------------------------------------------------------------*/
+
+void addanticycle(pathptr thispath, splineptr thisspline, short cycle)
+{
+   genericptr *ggen, *rgen;
+   splineptr otherspline;
+
+   if (areawin->pathedit == TANGENTS) {
+      for (ggen = thispath->plist; ggen < thispath->plist + thispath->parts;
+		ggen++)
+	 if (*ggen == (genericptr)thisspline) break;
+
+      if (*ggen != (genericptr)thisspline) return;	/* Something went wrong */
+
+      if (cycle == 1) {
+	 if (ggen > thispath->plist) {
+	    if (ELEMENTTYPE(*(ggen - 1)) == SPLINE) {
+	       addcycle(ggen - 1, 2, ANTIXY);
+	    }
+	 }
+	 else if (!(thispath->style & UNCLOSED)) {
+	    rgen = thispath->plist + thispath->parts - 1;
+	    if (ELEMENTTYPE(*rgen) == SPLINE) {
+	       otherspline = TOSPLINE(rgen);
+	       if (thisspline->ctrl[0].x == otherspline->ctrl[3].x &&
+			thisspline->ctrl[0].y == otherspline->ctrl[3].y)
+	          addcycle(rgen, 2, ANTIXY);
+	    }
+	 }
+      }
+      else if (cycle == 2) {	/* cycle should be only 1 or 2 */
+	 if (ggen < thispath->plist + thispath->parts - 1) {
+	    if (ELEMENTTYPE(*(ggen + 1)) == SPLINE) {
+	       addcycle(ggen + 1, 1, ANTIXY);
+	    }
+	 }
+	 else if (!(thispath->style & UNCLOSED)) {
+	    rgen = thispath->plist;
+	    if (ELEMENTTYPE(*rgen) == SPLINE) {
+	       otherspline = TOSPLINE(rgen);
+	       if (thisspline->ctrl[3].x == otherspline->ctrl[0].x &&
+			thisspline->ctrl[3].y == otherspline->ctrl[0].y)
+	          addcycle(rgen, 1, ANTIXY);
+	    }
+	 }
+      }
+   }
+}
+
+/*--------------------------------------------------------------*/
+/* Find the cycle numbered "cycle", and mark it as the		*/
+/* reference point.						*/
+/*--------------------------------------------------------------*/
+
+void makerefcycle(pointselect *cycptr, short cycle)
+{
+   pointselect *pptr, *sptr;
+
+   for (pptr = cycptr;; pptr++) {
+      if (pptr->flags & REFERENCE) {
+	 pptr->flags &= ~REFERENCE;
+	 break;
+      }
+      if (pptr->flags & LASTENTRY) break;
+   }
+
+   for (sptr = cycptr;; sptr++) {
+      if (sptr->number == cycle) {
+	 sptr->flags |= REFERENCE;
+	 break;
+      }
+      if (sptr->flags & LASTENTRY) break;
+   }
+
+   /* If something went wrong, revert to the original reference */
+
+   if (!(sptr->flags & REFERENCE)) {
+      pptr->flags |= REFERENCE;
+   }
+}
+      
+/* Original routine, used 1st entry as reference (deprecated) */
+
+void makefirstcycle(pointselect *cycptr, short cycle)
+{
+   pointselect *pptr, tmpp;
+
+   for (pptr = cycptr;; pptr++) {
+      if (pptr->number == cycle) {
+	 /* swap with first entry */
+	 tmpp = *cycptr;
+	 *cycptr = *pptr;
+	 *pptr = tmpp;
+	 if (cycptr->flags & LASTENTRY) {
+	    cycptr->flags &= ~LASTENTRY;
+	    pptr->flags |= LASTENTRY;
+	 }
+	 return;
+      }
+      if (pptr->flags & LASTENTRY) break;
+   }
+}
+
+/*------------------------------------------------------------------------------*/
+/* Advance a cycle (point) selection from value "cycle" to value "newvalue"	*/
+/* If "newvalue" is < 0 then remove the cycle.					*/
+/*										*/
+/* If there is only one cycle point on the element, then advance its point	*/
+/* number.  If there are multiple points on the element, then change the	*/
+/* reference point by moving the last item in the list to the front.		*/
+/*------------------------------------------------------------------------------*/
+
+void advancecycle(genericptr *pgen, short newvalue)
+{
+   polyptr ppoly;
+   arcptr parc;
+   splineptr pspline;
+   labelptr plabel;
+   pointselect *pptr, *endptr, *fcycle, **cycptr, tmpcyc;
+   short cycles = 0;
+
+   if (newvalue < 0) {
+      removecycle(pgen);
+      return;
+   }
+
+   switch((*pgen)->type) {
+      case POLYGON:
+	 ppoly = TOPOLY(pgen);
+	 cycptr = &ppoly->cycle;
+	 break;
+      case ARC:
+	 parc = TOARC(pgen);
+	 cycptr = &parc->cycle;
+	 break;
+      case SPLINE:
+	 pspline = TOSPLINE(pgen);
+	 cycptr = &pspline->cycle;
+	 break;
+      case LABEL:
+         plabel = TOLABEL(pgen);
+	 cycptr = &plabel->cycle;
+	 break;
+   }
+   if (*cycptr == NULL) return;
+
+   /* Remove any cycles that have only X or Y flags set.	*/
+   /* "Remove" them by shuffling them to the end of the list,	*/
+   /* and marking the one in front as the last entry.		*/
+
+   for (endptr = *cycptr; !(endptr->flags & LASTENTRY); endptr++);
+   pptr = *cycptr;
+   while (pptr < endptr) {
+      if ((pptr->flags & (EDITX | EDITY)) != (EDITX | EDITY)) {
+	 tmpcyc = *endptr; 
+	 *endptr = *pptr;
+	 *pptr = tmpcyc;
+	 pptr->flags &= ~LASTENTRY;
+	 pptr->number = -1;
+	 endptr--;
+	 endptr->flags |= LASTENTRY;
+      }
+      else
+	 pptr++;
+   }
+
+   if (pptr->flags & LASTENTRY) {
+      if ((pptr->flags & (EDITX | EDITY)) != (EDITX | EDITY)) {
+	 pptr->flags &= ~LASTENTRY;
+	 pptr->number = -1;
+	 endptr--;
+	 endptr->flags |= LASTENTRY;
+      }
+   }
+
+   /* Now advance the cycle */
+
+   pptr = *cycptr;
+   if (pptr->flags & LASTENTRY) {
+      pptr->number = newvalue;
+   }
+   else {
+      fcycle = *cycptr;
+      for (pptr = fcycle + 1;; pptr++) {
+	 if (pptr->flags & (EDITX | EDITY))
+	    fcycle = pptr;
+	 if (pptr->flags & LASTENTRY) break;
+      }
+      makerefcycle(*cycptr, fcycle->number);
+   }
+}
+
+/*--------------------------------------*/
+/* Remove a cycle (point) selection	*/
+/*--------------------------------------*/
+
+void removecycle(genericptr *pgen)
+{
+   polyptr ppoly;
+   pathptr ppath;
+   arcptr parc;
+   splineptr pspline;
+   labelptr plabel;
+   pointselect *pptr, **cycptr = NULL;
+   genericptr *pathgen;
+
+   switch((*pgen)->type) {
+      case POLYGON:
+	 ppoly = TOPOLY(pgen);
+	 cycptr = &ppoly->cycle;
+	 break;
+      case ARC:
+	 parc = TOARC(pgen);
+	 cycptr = &parc->cycle;
+	 break;
+      case SPLINE:
+	 pspline = TOSPLINE(pgen);
+	 cycptr = &pspline->cycle;
+	 break;
+      case LABEL:
+         plabel = TOLABEL(pgen);
+	 cycptr = &plabel->cycle;
+	 break;
+      case PATH:
+         ppath = TOPATH(pgen);
+	 for (pathgen = ppath->plist; pathgen < ppath->plist + ppath->parts;
+			pathgen++)
+	    removecycle(pathgen);
+	 break;
+   }
+   if (cycptr == NULL) return;
+   if (*cycptr == NULL) return;
+   free(*cycptr);
+   *cycptr = NULL;
+}
+
+/*--------------------------------------*/
+/* Remove cycles from all parts of a	*/
+/* path other than the one passed	*/
+/*--------------------------------------*/
+
+void removeothercycles(pathptr ppath, genericptr pathpart)
+{
+   genericptr *pathgen;
+   for (pathgen = ppath->plist; pathgen < ppath->plist + ppath->parts;
+		pathgen++)
+      if (*pathgen != pathpart)
+	 removecycle(pathgen);
+}
+
 /*--------------------------------------*/
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
@@ -378,7 +793,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 		objinstptr selinst)
 {
    selection	*rselect = NULL;
-   short	*newselect;
+   /* short	*newselect; (jdk) */
    genericptr	*curgen;
    XPoint 	newboxpts[4];
    Boolean	selected;
@@ -391,12 +806,6 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 
    for (curgen = selobj->plist; curgen < selobj->plist + selobj->parts; curgen++) {
 
-      /* when selection element to attach to, don't select the element */
-      /* to be attached. . .					       */
-
-      if (areawin->attachto == TRUE && eventmode != COPY_MODE && eventmode !=
-	    MOVE_MODE && curgen == EDITPART) continue;
-
       selected = False;
 
       /* Check among polygons, arcs, and curves */
@@ -429,7 +838,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 
 	    selected = test_insideness(areawin->save.x, areawin->save.y,
 		newboxpts);
-	    if (selected) textpos = textend = 0;
+	    if (selected) areawin->textpos = areawin->textend = 0;
 	 }
       }
 
@@ -452,20 +861,18 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 	 genericptr *pathp;
 
 	 /* Accept path if any subcomponent of the path is accepted */
-	 /* Record which part was selected in the "subeditpart" variable */
 
  	 for (pathp = TOPATH(curgen)->plist; pathp < TOPATH(curgen)->plist +
 		TOPATH(curgen)->parts; pathp++)
 	    if (pathselect(pathp, SPLINE|ARC|POLYGON, range)) {
 	       selected = True;
-	       areawin->editsubpart = (short)(pathp - TOPATH(curgen)->plist);
 	       break;
 	    }
       }
 
       else if ((*curgen)->type == (class & OBJINST)) {
 
-	 objinstbbox(TOOBJINST(curgen), newboxpts, True);
+	 objinstbbox(TOOBJINST(curgen), newboxpts, range);
 
          /* Look for an intersect of the boundingbox and pointer position. */
          /* This is a simple matter of rotating the pointer position with  */
@@ -504,48 +911,51 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 /* select arc, curve, and polygon objects from a defined box area */
 /*----------------------------------------------------------------*/
 
-Boolean areaelement(genericptr *curgen)
+Boolean areaelement(genericptr *curgen, XPoint *boxpts, Boolean is_path, short level)
 {
    Boolean selected;
    pointlist    currentpt;
+   short	cycle;
 
    switch(ELEMENTTYPE(*curgen)) {
 
       case(ARC):
 	   /* check center of arcs */
 
-      	   selected = (TOARC(curgen)->position.x < areawin->save.x) &&
-	  	(TOARC(curgen)->position.x > areawin->origin.x) &&
-		(TOARC(curgen)->position.y < areawin->save.y) &&
-		(TOARC(curgen)->position.y > areawin->origin.y);
+	   selected = test_insideness(TOARC(curgen)->position.x,
+		TOARC(curgen)->position.y, boxpts);
 	   break;
 
       case(POLYGON):
 	   /* check each point of the polygons */
 
 	   selected = False;
+	   cycle = 0;
            for (currentpt = TOPOLY(curgen)->points; currentpt <
-		TOPOLY(curgen)->points + TOPOLY(curgen)->number; currentpt++) {
-              if ((currentpt->x < areawin->save.x) && (currentpt->x >
-	           areawin->origin.x) && (currentpt->y < areawin->save.y) &&
-	           (currentpt->y > areawin->origin.y)) {
+		TOPOLY(curgen)->points + TOPOLY(curgen)->number;
+		currentpt++, cycle++) {
+	      if (test_insideness(currentpt->x, currentpt->y, boxpts)) {
 	         selected = True;
-		 break;
+		 if (level == 0) addcycle(curgen, cycle, 0);
 	      }
-           }
+	   }
 	   break;
 
       case(SPLINE):
 	   /* check each control point of the spline */
 	
-           selected = ((TOSPLINE(curgen)->ctrl[0].x < areawin->save.x) && 
-	   	(TOSPLINE(curgen)->ctrl[0].x > areawin->origin.x) &&
-		(TOSPLINE(curgen)->ctrl[0].y < areawin->save.y) &&
-		(TOSPLINE(curgen)->ctrl[0].y > areawin->origin.y))
-		|| ((TOSPLINE(curgen)->ctrl[3].x < areawin->save.x) &&
-		(TOSPLINE(curgen)->ctrl[3].x > areawin->origin.x) &&
-		(TOSPLINE(curgen)->ctrl[3].y < areawin->save.y) &&
-		(TOSPLINE(curgen)->ctrl[3].y > areawin->origin.y));
+	   selected = False;
+	   if (test_insideness(TOSPLINE(curgen)->ctrl[0].x,
+			TOSPLINE(curgen)->ctrl[0].y, boxpts)) {
+	      selected = True;
+	      if (level == 0) addcycle(curgen, 0, 0);
+	   }
+
+	   if (test_insideness(TOSPLINE(curgen)->ctrl[3].x,
+			TOSPLINE(curgen)->ctrl[3].y, boxpts)) {
+	      selected = True;
+	      if (level == 0) addcycle(curgen, 3, 0);
+	   }
 	   break;
    }
    return selected;
@@ -555,55 +965,45 @@ Boolean areaelement(genericptr *curgen)
 /* select all objects from a defined box area */
 /*--------------------------------------------*/
 
-void selectarea()
+Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 {
    short	*newselect;
    genericptr   *curgen, *pathgen;
    Boolean	selected;
    stringpart	*strptr;
-   int		locpos;
+   int		locpos, cx, cy, hwidth, hheight;
+   objinstptr	curinst;
+   XPoint	newboxpts[4];
 
-   /* put points of area bounding box into proper order */
+   if (selobj == topobject) {
+      areawin->textpos = areawin->textend = 0;
+   } 
 
-   if (areawin->origin.y > areawin->save.y) {
-      short tmp;
-      tmp = areawin->origin.y;
-      areawin->origin.y = areawin->save.y;
-      areawin->save.y = tmp;
-   }
-   if (areawin->origin.x > areawin->save.x) {
-      short tmp;
-      tmp = areawin->origin.x;
-      areawin->origin.x = areawin->save.x;
-      areawin->save.x = tmp;
-   }
-
-   textpos = textend = 0;
-   for (curgen = topobject->plist; curgen < topobject->plist
-	+ topobject->parts; curgen++) {
+   for (curgen = selobj->plist; curgen < selobj->plist + selobj->parts; curgen++) {
 
       /* apply the selection filter */
       if (!((*curgen)->type & areawin->filter)) continue;
 
       switch(ELEMENTTYPE(*curgen)) {
 	case(OBJINST):
-           /* check for instance of object center point inside area box */
-           selected = (TOOBJINST(curgen)->position.x < areawin->save.x) &&
-		(TOOBJINST(curgen)->position.x > areawin->origin.x) &&
-		(TOOBJINST(curgen)->position.y < areawin->save.y) &&
-		(TOOBJINST(curgen)->position.y > areawin->origin.y);
-	   break;
+	    curinst = TOOBJINST(curgen);
+
+	    /* An object instance is selected if any part of it is	*/
+	    /* selected on a recursive area search.			*/
+
+            InvTransformPoints(boxpts, newboxpts, 4, curinst->position,
+			curinst->scale, curinst->rotation);
+            selected = selectarea(curinst->thisobject, newboxpts, level + 1);
+	    break;
 
 	case(GRAPHIC):
            /* check for graphic image center point inside area box */
-           selected = (TOGRAPHIC(curgen)->position.x < areawin->save.x) &&
-		(TOGRAPHIC(curgen)->position.x > areawin->origin.x) &&
-		(TOGRAPHIC(curgen)->position.y < areawin->save.y) &&
-		(TOGRAPHIC(curgen)->position.y > areawin->origin.y);
+	   selected = test_insideness(TOGRAPHIC(curgen)->position.x,
+		TOGRAPHIC(curgen)->position.y, boxpts);
 	   break;
 
         case(LABEL): {
-	   XPoint bboxpts[4], newboxpts[4], adj;
+	   XPoint adj;
 	   labelptr slab = TOLABEL(curgen);
 	   short j, state, isect, tmpl1, tmpl2;
 	   TextExtents tmpext;
@@ -613,14 +1013,11 @@ void selectarea()
 	   /* Ignore temporary labels created by the netlist generator */
 	   if (slab->string->type != FONT_NAME) break;
 
-	   /* create a 4-point polygon out of the select box information */
-	   bboxpts[0].x = bboxpts[3].x = areawin->origin.x;
-	   bboxpts[0].y = bboxpts[1].y = areawin->origin.y;
-	   bboxpts[1].x = bboxpts[2].x = areawin->save.x;
-	   bboxpts[2].y = bboxpts[3].y = areawin->save.y;
+	   /* Ignore info and pin labels that are not on the top level */
+	   if ((selobj != topobject) && (slab->pin != False)) break;
 
    	   /* translate select box into the coordinate system of the label */
-	   InvTransformPoints(bboxpts, newboxpts, 4, slab->position,
+	   InvTransformPoints(boxpts, newboxpts, 4, slab->position,
 		  slab->scale, slab->rotation);
 
 	   if (slab->pin) {
@@ -629,7 +1026,7 @@ void selectarea()
 			&(newboxpts[j].y), -1);
 	   }
 
-	   tmpext = ULength(slab->string, areawin->topinstance, 0.0, 0, NULL);
+	   tmpext = ULength(slab, areawin->topinstance, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
 			tmpext.width : tmpext.width >> 1) : 0);
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
@@ -645,7 +1042,7 @@ void selectarea()
 	      if (locpos < 0) continue;	  /* only look at printable characters */
 	      if (strptr->type == RETURN) tmpl2 = 0;
 	      tmpl1 = tmpl2;
-	      tmpext = ULength(slab->string, areawin->topinstance, 0.0, j + 1, NULL);
+	      tmpext = ULength(slab, areawin->topinstance, j + 1, NULL);
 	      tmpl2 = tmpext.width;
 	      isect = test_insideness(((tmpl1 + tmpl2) >> 1) - adj.x,
 			(tmpext.base + (BASELINE >> 1)) - adj.y, newboxpts);
@@ -655,20 +1052,20 @@ void selectarea()
 		 if (isect) {
 		    state = 1;
 		    selected = True;
-		    textend = j;
-		    if ((textend > 1) && strptr->type != TEXT_STRING)
-		       textend--;
+		    areawin->textend = j;
+		    if ((areawin->textend > 1) && strptr->type != TEXT_STRING)
+		       areawin->textend--;
 		 }
 	      }
 	      else {
 		 if (!isect) {
-		    textpos = j;
+		    areawin->textpos = j;
 		    state = 2;
 		    break;
 		 }
 	      }
 	   }
-	   if (state == 1) textpos = j;   /* selection goes to end of string */
+	   if (state == 1) areawin->textpos = j;   /* selection goes to end of string */
 	   } break;
 	   
 	case(PATH):
@@ -677,15 +1074,19 @@ void selectarea()
 	   selected = False;
 	   for (pathgen = TOPATH(curgen)->plist; pathgen < TOPATH(curgen)->plist
 		  + TOPATH(curgen)->parts; pathgen++) {
-	      if (areaelement(pathgen)) selected = True;
+	      if (areaelement(pathgen, boxpts, True, level)) selected = True;
 	   }
 	   break;
 
 	default:
-	   selected = areaelement(curgen);
+	   selected = areaelement(curgen, boxpts, False, level);
 	   break;
       }
 
+      /* on recursive searches, return as soon as we find something */
+
+      if ((selobj != topobject) && selected) return TRUE;
+
       /* check if this part has already been selected */
 
       if (selected)
@@ -701,13 +1102,14 @@ void selectarea()
          *newselect = (short)(curgen - topobject->plist);
       }
    }
+   if (selobj != topobject) return FALSE;
    setoptionmenu();
 
    /* if none or > 1 label has been selected, cancel any textpos placement */
 
    if (!checkselect(LABEL) || areawin->selects != 1 ||
 	(areawin->selects == 1 && SELECTTYPE(areawin->selectlist) != LABEL)) {
-      textpos = textend = 0;
+      areawin->textpos = areawin->textend = 0;
    }
 
    /* Register the selection as an undo event */
@@ -716,6 +1118,13 @@ void selectarea()
 
    /* Drawing of selected objects will take place when drawarea() is */
    /* executed after the button release.			     */
+
+#ifdef TCL_WRAPPER
+   if (xobjs.suspend < 0)
+      XcInternalTagCall(xcinterp, 2, "select", "here");
+#endif
+
+   return selected;
 }
 
 /*------------------------*/
@@ -739,8 +1148,9 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
 void draw_all_selected()
 {
    int j;
-
+   
    if (areawin->hierstack != NULL) return;
+
    for (j = 0; j < areawin->selects; j++)
       gendrawselected(areawin->selectlist + j, topobject, areawin->topinstance);
 }
@@ -751,21 +1161,19 @@ void draw_all_selected()
 
 void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 {
-   short *lastselect;
-   int i;
+   short saveselects;
 
    if (areawin->selects == 0) return;
    else if (areawin->hierstack != NULL) return;
 
-   /* reset the graphics state */
+   saveselects = areawin->selects;
 
+   /* reset the graphics state */
    SetFunction(dpy, areawin->gc, GXcopy);
 
-   for (i = 0; i < areawin->selects; i++) {
-      lastselect = areawin->selectlist + i;
-      XTopSetForeground(SELTOCOLOR(lastselect)); 
-      geneasydraw(*lastselect, SELTOCOLOR(lastselect), thisobj, thisinst);
-   }
+   areawin->selects = 0;
+   drawarea(NULL, NULL, NULL);
+   areawin->selects = saveselects;
 }
 
 /*----------------------------------------------------------------------*/
@@ -810,10 +1218,19 @@ void freeselects()
 void clearselects_noundo()
 {
    if (areawin->selects > 0) {
+      reset_cycles();
       freeselects();
-      setallstylemarks(areawin->style);
-      setcolormark(areawin->color);
-      setdefaultfontmarks();
+      if (xobjs.suspend < 0) {
+         setallstylemarks(areawin->style);
+         setcolormark(areawin->color);
+         setdefaultfontmarks();
+	 setparammarks(NULL);
+      }
+
+#ifdef TCL_WRAPPER
+      if (xobjs.suspend < 0)
+         XcInternalTagCall(xcinterp, 2, "unselect", "all");
+#endif
    }
 }
 
@@ -837,8 +1254,10 @@ void clearselects()
 
 void unselect_all()
 {
-   draw_normal_selected(topobject, areawin->topinstance);
+   if (xobjs.suspend < 0)
+      draw_normal_selected(topobject, areawin->topinstance);
    clearselects();
+
 }
 
 /*----------------------------------------------------------------------*/
@@ -854,8 +1273,7 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 {
    selection *rselect, *rcheck, *lastselect;
    genericptr rgen;
-   short i, *selectobj, numselect, origselect;
-   short rclass;
+   short i;
    objectptr selobj;
    objinstptr selinst;
    XPoint savesave, tmppt;
@@ -902,12 +1320,8 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 			"in new object is (%d, %d)\n",
 			savesave.x, savesave.y,
 			areawin->save.x, areawin->save.y); */
-         UPushCTM();
-         UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
-	 rclass = class;
-	 if ((class & (~OBJINST)) == 0) class = ALL_TYPES;
-         rcheck = recurselect(class, recmode, &selnew);
-         UPopCTM();
+
+         rcheck = recurselect(ALL_TYPES, recmode, &selnew);
          areawin->save.x = savesave.x;
          areawin->save.y = savesave.y;
 
@@ -975,7 +1389,7 @@ void startselect()
 void trackselarea()
 {
    XPoint newpos;
-   u_int  nullui;
+   /* u_int  nullui; (jdk) */
 
    newpos = UGetCursorPos();
    if (newpos.x == areawin->save.x && newpos.y == areawin->save.y) return;
@@ -994,7 +1408,6 @@ void trackselarea()
 void trackrescale()
 {
    XPoint newpos;
-   u_int  nullui;
 
    newpos = UGetCursorPos();
    if (newpos.x == areawin->save.x && newpos.y == areawin->save.y) return;
@@ -1059,6 +1472,151 @@ Boolean compareselection(selection *sa, selection *sb)
    return (match == sa->selects) ? True : False;
 }
 
+/*----------------------------------------------------------------------*/
+/* Add pin cycles connected to selected labels				*/
+/*----------------------------------------------------------------------*/
+
+void label_connect_cycles(labelptr thislab)
+{
+   genericptr *pgen;
+   Boolean is_selected;
+   XPoint *testpt;
+   polyptr cpoly;
+   short *stest, cycle;
+
+   if (thislab->pin == LOCAL || thislab->pin == GLOBAL) {
+      for (pgen = topobject->plist; pgen < topobject->plist +
+			topobject->parts; pgen++) {
+	 /* Ignore any wires that are already selected */
+	 is_selected = FALSE;
+	 for (stest = areawin->selectlist; stest < areawin->selectlist +
+			areawin->selects; stest++) {
+	    if (SELTOGENERIC(stest) == *pgen) {
+	       is_selected = TRUE;
+	       break;
+	    }
+	 }
+	 if (ELEMENTTYPE(*pgen) == POLYGON) {
+	    cpoly = TOPOLY(pgen);
+	    if (!is_selected) {
+	       cycle = 0;
+	       for (testpt = cpoly->points; testpt < cpoly->points +
+			cpoly->number; testpt++) {
+		  if (testpt->x == thislab->position.x
+			&& testpt->y == thislab->position.y) {
+		     addcycle(pgen, cycle, 0);
+		     break;
+		  }
+		  else
+		     cycle++;
+	       }
+	    }
+	    else {
+	       /* Make sure that this polygon's cycle is not set! */
+	       removecycle(pgen);
+	    }
+	 }
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Add pin cycles connected to selected instances			*/
+/*----------------------------------------------------------------------*/
+
+void inst_connect_cycles(objinstptr thisinst)
+{
+   genericptr *ggen, *pgen;
+   Boolean is_selected;
+   XPoint refpoint, *testpt;
+   labelptr clab;
+   polyptr cpoly;
+   short *stest, cycle;
+   objectptr thisobj = thisinst->thisobject;
+
+   for (ggen = thisobj->plist; ggen < thisobj->plist + thisobj->parts; ggen++) {
+      if (ELEMENTTYPE(*ggen) == LABEL) {
+	 clab = TOLABEL(ggen);
+	 if (clab->pin == LOCAL || clab->pin == GLOBAL) {
+	    ReferencePosition(thisinst, &clab->position, &refpoint);
+	    for (pgen = topobject->plist; pgen < topobject->plist +
+			topobject->parts; pgen++) {
+	       /* Ignore any wires that are already selected */
+	       is_selected = FALSE;
+	       for (stest = areawin->selectlist; stest < areawin->selectlist +
+			areawin->selects; stest++) {
+		  if (SELTOGENERIC(stest) == *pgen) {
+		     is_selected = TRUE;
+		     break;
+		  }
+	       }
+	       if (ELEMENTTYPE(*pgen) == POLYGON) {
+		  cpoly = TOPOLY(pgen);
+	          if (!is_selected) {
+		     cycle = 0;
+		     for (testpt = cpoly->points; testpt < cpoly->points +
+				cpoly->number; testpt++) {
+		        if (testpt->x == refpoint.x && testpt->y == refpoint.y) {
+			   addcycle(pgen, cycle, 0);
+			   break;
+		        }
+		        else
+		           cycle++;
+		     }
+		  }
+	          else {
+		     /* Make sure that this polygon's cycle is not set! */
+		     removecycle(pgen);
+		  }
+	       }
+	    }
+	 }
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Select connected pins on all selected object instances and labels	*/
+/*----------------------------------------------------------------------*/
+
+void select_connected_pins()
+{
+   short *selptr;
+   objinstptr selinst;
+   labelptr sellab;
+
+   if (!areawin->pinattach) return;
+
+   for (selptr = areawin->selectlist; selptr < areawin->selectlist +
+		areawin->selects; selptr++) {
+      switch (SELECTTYPE(selptr)) {
+	 case LABEL:
+            sellab = SELTOLABEL(selptr);
+            label_connect_cycles(sellab);
+	    break;
+	 case OBJINST:
+            selinst = SELTOOBJINST(selptr);
+            inst_connect_cycles(selinst);
+	    break;
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Reset all polygon cycles flagged during a move (polygon wires	*/
+/* connected to pins of an object instance).				*/
+/*----------------------------------------------------------------------*/
+
+void reset_cycles()
+{
+   polyptr cpoly;
+   genericptr *pgen;
+
+   for (pgen = topobject->plist; pgen < topobject->plist +
+			topobject->parts; pgen++)
+      removecycle(pgen);
+}
+
 /*----------------------------------------------------------------------*/
 /* Recursive selection mechanism					*/
 /*----------------------------------------------------------------------*/
@@ -1066,7 +1624,7 @@ Boolean compareselection(selection *sa, selection *sb)
 short *recurse_select_element(short class, u_char mode) {
    pushlistptr seltop, nextptr;
    selection *rselect;
-   short *newselect, *desel, localpick;
+   short *newselect, localpick; /* *desel, (jdk) */
    static short pick = 0;
    static selection *saveselect = NULL;
    int i, j, k, ilast, jlast;
@@ -1179,5 +1737,10 @@ short *recurse_select_element(short class, u_char mode) {
    freeselection(saveselect);
    saveselect = rselect;
 
+#ifdef TCL_WRAPPER
+   if (xobjs.suspend < 0)
+      XcInternalTagCall(xcinterp, 2, "select", "here");
+#endif
+
    return areawin->selectlist;
 }
