@@ -23,6 +23,10 @@
 #include <X11/StringDefs.h>
 #endif
 
+#ifndef HAVE_VFORK
+#define vfork fork
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Local includes							*/
 /*----------------------------------------------------------------------*/
