@@ -8,10 +8,19 @@
 /*----------------------------------------------------------------------*/
 
 #ifndef HAVE_U_CHAR
-typedef unsigned char	u_char;
-typedef unsigned short	u_short;
-typedef unsigned int	u_int;
-typedef unsigned long	u_long;
+typedef unsigned char       u_char;
+typedef unsigned short      u_short;
+typedef unsigned int        u_int;
+typedef unsigned long       u_long;
+typedef unsigned long long  u_long_long;
+#endif
+
+#ifdef XC_WIN32
+#ifdef TCL_WRAPPER
+#include "tkwin32.h"
+#else
+#include "xcwin32.h"
+#endif
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -70,6 +79,18 @@ typedef unsigned long	u_long;
 /*----------------------------------------------------------------------*/
 /* Deal with 32/64 bit processors based on autoconf results.		*/
 /*----------------------------------------------------------------------*/
+#ifndef SIZEOF_VOID_P
+#error "SIZEOF_VOID_P undefined!"
+#endif
+#ifndef SIZEOF_UNSIGNED_INT
+#error "SIZEOF_UNSIGNED_INT undefined!"
+#endif
+#ifndef SIZEOF_UNSIGNED_LONG
+#error "SIZEOF_UNSIGNED_LONG undefined!"
+#endif
+#ifndef SIZEOF_UNSIGNED_LONG_LONG
+#error "SIZEOF_UNSIGNED_LONG_LONG undefined!"
+#endif
 
 #if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
 #define Number(a)		(void *)((u_int) a)
@@ -77,6 +98,9 @@ typedef u_int pointertype;
 #elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG
 #define Number(a)		(void *)((u_long) a)
 typedef u_long pointertype;
+#elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG_LONG
+#define Number(a)		(void *)((u_long_long) a)
+typedef u_long_long pointertype;
 #else
 ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #endif
@@ -99,7 +123,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define ARC		0x08
 #define SPLINE  	0x10
 #define PATH		0x20
-#define GRAPHIC		0x40	/* reserved; unused, for now.		*/
+#define GRAPHIC		0x40
 #define ARRAY		0x80	/* reserved; unused, for now.		*/
 
 #define REMOVE_TAG	0x100	/* element to be removed from netlist	*/
@@ -119,8 +143,10 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define XtnSetArg(a,b)  	XtSetArg(wargs[n], a, b); n++
 #define abs(a)			((a) < 0 ? -(a) : (a))
 #define sign(a)			((a) <= 0 ? -1 : 1)
+#ifndef min
 #define max(a,b)		((a) < (b) ? (b) : (a))
 #define min(a,b)		((a) < (b) ? (a) : (b))
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Lengthier define constructs						*/
@@ -136,8 +162,8 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
 
-#define ENDPART		topobject->plist + topobject->parts
-#define EDITPART	topobject->plist + areawin->editpart
+#define ENDPART		topobject->plist + topobject->parts - 1
+#define EDITPART	topobject->plist + *areawin->selectlist
 
 /* Type checks (argument "a" is type genericptr) */
 
@@ -189,36 +215,43 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	PLIST_INCR(b); \
 	a = (polyptr *)b->plist + b->parts; \
 	*a = (polyptr) malloc(sizeof(polygon)); \
+	b->parts++; \
 	(*a)->type = POLYGON
 #define NEW_LABEL(a,b) \
 	PLIST_INCR(b); \
 	a = (labelptr *)b->plist + b->parts; \
 	*a = (labelptr) malloc(sizeof(label)); \
+	b->parts++; \
 	(*a)->type = LABEL
 #define NEW_OBJINST(a,b) \
 	PLIST_INCR(b); \
 	a = (objinstptr *)b->plist + b->parts; \
 	*a = (objinstptr) malloc(sizeof(objinst)); \
+	b->parts++; \
 	(*a)->type = OBJINST
 #define NEW_ARC(a,b) \
 	PLIST_INCR(b); \
 	a = (arcptr *)b->plist + b->parts; \
 	*a = (arcptr) malloc(sizeof(arc)); \
+	b->parts++; \
 	(*a)->type = ARC
 #define NEW_SPLINE(a,b) \
 	PLIST_INCR(b); \
 	a = (splineptr *)b->plist + b->parts; \
 	*a = (splineptr) malloc(sizeof(spline)); \
+	b->parts++; \
 	(*a)->type = SPLINE
 #define NEW_PATH(a,b) \
 	PLIST_INCR(b); \
 	a = (pathptr *)b->plist + b->parts; \
 	*a = (pathptr) malloc(sizeof(path)); \
+	b->parts++; \
 	(*a)->type = PATH
 #define NEW_GRAPHIC(a,b) \
 	PLIST_INCR(b); \
 	a = (graphicptr *)b->plist + b->parts; \
 	*a = (graphicptr) malloc(sizeof(graphic)); \
+	b->parts++; \
 	(*a)->type = GRAPHIC
 
 /*----------------------------------------------------------------------*/
@@ -262,6 +295,8 @@ typedef struct {
 #define INTSEGS		(SPLINESEGS - 2)
 #define LASTSEG		(SPLINESEGS - 3)
 
+#define NOFILENAME	(char *)(-1)
+
 #define FONTHEIGHT(y)	 (y->ascent + y->descent + 6) 
 #define ROWHEIGHT	 FONTHEIGHT(appdata.xcfont)
 #define FILECHARASCENT   (appdata.filefont->ascent)
@@ -294,11 +329,12 @@ enum editmode {
   COPY_MODE,		/* In the process of copying elements */
   PAN_MODE,		/* In the process of panning to follow the cursor */
   SELAREA_MODE,		/* Area selection box */
+  PENDING_MODE,		/* Temporary mode to select without drawing selection */
   RESCALE_MODE,		/* Interactive element rescaling box */
   CATALOG_MODE,		/* On a library page, library directory, or page directory */
   CATTEXT_MODE,		/* Editing an existing object name in the library */
-  FONTCAT_MODE,		/* Accessing the font character page from TEXT2_MODE */
-  EFONTCAT_MODE,	/* Accessing the font character page from TEXT3_MODE */
+  FONTCAT_MODE,		/* Accessing the font character page from TEXT_MODE */
+  EFONTCAT_MODE,	/* Accessing the font character page from ETEXT_MODE */
   TEXT_MODE,		/* Creating a new label */
   WIRE_MODE,		/* Creating a new polygon (wire) */
   BOX_MODE,		/* Creating a new box */
@@ -309,6 +345,7 @@ enum editmode {
   EARC_MODE,		/* Editing an existing arc */
   ESPLINE_MODE,		/* Editing an existing spline */
   EPATH_MODE,		/* Editing an existing path */
+  EINST_MODE,		/* Editing an instance (from the level above) */
   ASSOC_MODE,		/* Choosing an associated schematic or symbol */
   CATMOVE_MODE		/* Moving objects in or between libraries */
 };
@@ -365,7 +402,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define PARAM_START	17  /* bounds a parameter; data = param key	*/
 #define PARAM_END	18  /* bounds a parameter; no data 		*/
 
-/* Actions translated to keystates */
+/* Actions translated to keystates (numbering continues from above) */
 
 #define TEXT_RETURN	19
 #define TEXT_HOME	20
@@ -401,12 +438,19 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Library types							*/
 /*----------------------------------------------------------------------*/
 
+#define FONTENCODING   -1  /* Used only by libopen() */
 #define FONTLIB		0
 #define PAGELIB		1
 #define LIBLIB		2
 #define LIBRARY		3
 #define USERLIB		(xobjs.numlibs + LIBRARY - 1)
 
+/*----------------------------------------------------------------------*/
+/* Object instance styles						*/
+/*----------------------------------------------------------------------*/
+
+#define LINE_INVARIANT 1	/* Linewidth is invariant w.r.t. scale */
+
 /*----------------------------------------------------------------------*/
 /* Box styles								*/
 /*----------------------------------------------------------------------*/
@@ -421,24 +465,40 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define STIP1 64
 #define STIP2 128
 #define FILLSOLID 224  /* = 32 + 64 + 128 */
+#ifdef OPAQUE
+#undef OPAQUE
+#endif
 #define OPAQUE 256
 #define BBOX 512
 #define SQUARECAP 1024
+#define CLIPMASK  2048
 
 /*----------------------------------------------------------------------*/
 /* Box edit styles							*/
 /*----------------------------------------------------------------------*/
 
+#define NONE  0
 #define MANHATTAN 1
 #define RHOMBOIDX 2
 #define RHOMBOIDY 4
 #define RHOMBOIDA 8
 
-#define NONE  0
-#define LASTX 1
-#define NEXTX 2
-#define LASTY 4
-#define NEXTY 8
+/*----------------------------------------------------------------------*/
+/* Path edit styles							*/
+/*----------------------------------------------------------------------*/
+
+#define TANGENTS  1		/* (NORMAL = 0) */
+
+/*----------------------------------------------------------------------*/
+/* Cycle points (selected points) (flag bits, can be OR'd together)	*/
+/*----------------------------------------------------------------------*/
+
+#define EDITX 	  0x01
+#define EDITY	  0x02
+#define LASTENTRY 0x04
+#define PROCESS	  0x08
+#define REFERENCE 0x10
+#define ANTIXY	  0x20		/* For matched-tangent curves */
 
 /*----------------------------------------------------------------------*/
 /* Arc creation and edit styles						*/
@@ -463,6 +523,8 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define TRIVIAL 2		/* Symbol as non-schematic element */
 #define SYMBOL 3		/* Symbol associated with a schematic */
 #define FUNDAMENTAL 4		/* Standalone symbol */
+#define NONETWORK 5		/* Do not netlist this object */
+#define GLYPH 6			/* Symbol is a font glyph */
 
 #define LOCAL 1
 #define GLOBAL 2
@@ -484,8 +546,9 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define INDEPENDENT	0
 #define DEPENDENT	1
 #define TOTAL_PAGES	2
-#define PAGE_DEPEND	3
-#define ALL_PAGES	4
+#define LINKED_PAGES	3
+#define PAGE_DEPEND	4
+#define ALL_PAGES	5
 
 /*----------------------------------------------------------------------*/
 /* Color scheme styles (other than NORMAL)				*/
@@ -514,7 +577,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define DEFAULTCURSOR	(*areawin->defaultcursor)
 
 /*----------------------------------------------------------------------*/
-/* selection-mechanism structures					*/
+/* integer and floating-point coordinate list structures		*/
 /*----------------------------------------------------------------------*/
 
 typedef XPoint* pointlist;
@@ -524,22 +587,25 @@ typedef XfPoint* fpointlist;
 /* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
 
-#define P_NUMERIC	0	/* uncommitted numeric parameter */
-#define P_SUBSTRING	1
-#define P_POSITION_X	2
-#define P_POSITION_Y	3
-#define P_STYLE		4
-#define P_JUSTIFY	5
-#define P_ANGLE1	6
-#define P_ANGLE2	7
-#define P_RADIUS	8
-#define P_MINOR_AXIS	9
-#define P_ROTATION	10
-#define P_SCALE		11
-#define P_LINEWIDTH	12
-#define P_COLOR		13
-#define P_EXPRESSION	14
-#define P_POSITION	15	/* mode only, not a real parameter */
+enum paramwhich {
+   P_NUMERIC = 0,	/* uncommitted numeric parameter */
+   P_SUBSTRING,
+   P_POSITION_X,
+   P_POSITION_Y,
+   P_STYLE,
+   P_JUSTIFY,
+   P_ANGLE1,
+   P_ANGLE2,
+   P_RADIUS,
+   P_MINOR_AXIS,
+   P_ROTATION,
+   P_SCALE,
+   P_LINEWIDTH,
+   P_COLOR,
+   P_EXPRESSION,
+   P_POSITION, 		/* mode only, not a real parameter */
+   NUM_PARAM_TYPES
+};
 
 /*----------------------------------------------------------------------*/
 /* Labels are constructed of strings and executables 			*/
@@ -597,13 +663,17 @@ typedef struct _eparam *eparamptr;
 
 typedef struct _eparam {
    char *	key;	    /* name of the parameter */
+   u_char	flags;	    /* namely, bit declaring an indirect parameter */
    union {
       int	pointno;    /* point number in point array, for polygons */
+      short	pathpt[2];  /* element number and point number, for paths */
       char	*refkey;    /* parameter reference key, for instances */
    } pdata;
    eparamptr	next;	    /* next parameter in linked list */
 } eparam;
 
+#define P_INDIRECT	0x01	/* indirect parameter indicator */
+
 /*----------------------------------------------------------------------*/
 /* Generic element type	is a superset of all elements.			*/	
 /*----------------------------------------------------------------------*/
@@ -614,6 +684,21 @@ typedef struct {
    eparamptr	passed;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
 
+/*----------------------------------------------------------------------*/
+/* selection-mechanism structures					*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   short number;
+   genericptr *element;
+   short *idx;
+} uselection;
+
+typedef struct {
+   short number;
+   u_char flags;
+} pointselect;
+
 /*----------------------------------------------------------------------*/
 /* Bounding box								*/
 /*----------------------------------------------------------------------*/
@@ -633,6 +718,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   u_short	style;
    XPoint	position;
    short	rotation;
    float	scale;
@@ -658,6 +744,7 @@ typedef struct _pushlist *pushlistptr;
 
 typedef struct _pushlist {
    objinstptr	thisinst;
+   char		*clientdata;	/* general-purpose record */
    pushlistptr	next;
 } pushlist;
 
@@ -686,6 +773,8 @@ typedef struct {
    float	  scale;
    XImage	  *source;	/* source data */
    XImage	  *target;	/* target (scaled) data */
+   short	  trot;		/* target rotation */
+   float	  tscale;	/* target scale (0 = uninitialized) */
    Pixmap	  clipmask;	/* clipmask for non-manhattan rotations */
    Boolean	  valid;	/* does target need to be regenerated? */
 } graphic, *graphicptr;
@@ -698,6 +787,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    XPoint	position;
    short	rotation;
    float	scale;
@@ -716,6 +806,7 @@ typedef struct {
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    short	number;
    pointlist	points;
 } polygon, *polyptr;
@@ -730,6 +821,7 @@ typedef struct {
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    XPoint	ctrl[4];
    /* the following are for rendering only */
    XfPoint	points[INTSEGS];
@@ -745,9 +837,10 @@ typedef struct {
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    short	radius; 	/* x-axis radius */
    short	yaxis;		/* y-axis radius */
-   float	angle1;
+   float	angle1;		/* endpoint angles, in degrees */
    float	angle2;
    XPoint	position;
    /* the following are for rendering only */
@@ -966,7 +1059,7 @@ typedef struct _matrix {
 /* button tap/press definitions 					*/
 /*----------------------------------------------------------------------*/
 
-#define PRESSTIME	150	/* milliseconds of push to be a "press" */
+#define PRESSTIME	200	/* milliseconds of push to be a "press" */
 
 /*----------------------------------------------------------------------*/
 /* object name alias structures						*/
@@ -1005,7 +1098,7 @@ typedef struct _alias {
 #define xcScreen			Tk_Screen
 #define xcParent			Tk_Parent
 #define xcDispatchEvent(a)		Tk_HandleEvent(a)
-#define xcAddTimeOut(a, b, c, d)	Tcl_CreateTimerHandler(b, c, d)
+#define xcAddTimeOut(a, b, c, d)	Tcl_CreateTimerHandler((int)(b), c, d)
 #define xcRemoveTimeOut			Tcl_DeleteTimerHandler
 #define xcTimeOutProc			Tcl_TimerProc *
 #define xcIntervalId			Tcl_TimerToken
@@ -1029,7 +1122,6 @@ typedef struct _alias {
 #ifndef ClientData
    #define ClientData			XtPointer
 #endif
-
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -1078,26 +1170,23 @@ typedef struct {
    Pixel	globalcolor, localcolor, infocolor, ratsnestcolor;
 
    /* non-schematic layout color(s) */
-   Pixel	bboxpix;
+   Pixel	bboxpix, clipcolor;
 
    /* color scheme 1 */
    Pixel	fg, bg;
    Pixel	gridpix, snappix, selectpix, axespix;
-   Pixel	buttonpix, querypix, filterpix;
-   Pixel	auxpix, barpix, parampix;
+   Pixel	buttonpix, filterpix, auxpix, barpix, parampix;
 
    /* color scheme 2 */
-
    Pixel	fg2, bg2;
    Pixel	gridpix2, snappix2, selectpix2, axespix2;
-   Pixel	buttonpix2, querypix2, filterpix2;
-   Pixel	auxpix2, barpix2, parampix2;
+   Pixel	buttonpix2, auxpix2, parampix2;
 
    int		width, height, timeout;
-   XFontStruct	*helpfont, *filefont;
+   XFontStruct	*filefont;
 
 #ifndef TCL_WRAPPER
-   XFontStruct	*xcfont, *textfont, *titlefont;
+   XFontStruct	*xcfont, *textfont, *titlefont, *helpfont;
 #endif
 
 } ApplicationData, *ApplicationDataPtr;
@@ -1116,9 +1205,12 @@ typedef struct {
 #define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
 	areawin->gctype = z
 
-#define XcSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
+#define XcTopSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
+#define XcSetForeground(z) SetForeground(dpy, areawin->gc, z); \
+	areawin->gccolor = z
+
 #define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
@@ -1197,7 +1289,7 @@ enum {
    XCF_Text_Split /* 24 */,	XCF_Text_Home /* 25 */,
    XCF_Text_End /* 26 */,	XCF_Linebreak /* 27 */,
    XCF_Parameter /* 28 */,	XCF_Edit_Param /* 29 */,
-   XCF_Edit_Break /* 30 */,	XCF_Edit_Delete /* 31 */,
+   XCF_ChangeStyle /* 30 */,	XCF_Edit_Delete /* 31 */,
    XCF_Edit_Insert /* 32 */,	XCF_Edit_Append /* 33 */,
    XCF_Edit_Next /* 34 */,	XCF_Attach /* 35 */,
    XCF_Next_Library /* 36 */,	XCF_Library_Directory /* 37 */,
@@ -1229,15 +1321,15 @@ enum {
    XCF_Exit /* 88 */,		XCF_Netlist /* 89 */,
    XCF_Swap /* 90 */,		XCF_Pin_Label /* 91 */,
    XCF_Pin_Global /* 92 */,	XCF_Info_Label /* 93 */,
-   XCF_Graphic	/* 94 */,	XCF_Rescale /* 95 */,
-   XCF_SelectBox /* 96 */,	XCF_Connectivity /* 97 */,
-   XCF_Continue_Copy /* 98 */,	XCF_Continue_Element /* 99 */,
-   XCF_Finish_Copy /* 100 */,	XCF_Finish_Element /* 101 */,
-   XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
-   XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
-   XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
-   XCF_Color /* 108 */,		XCF_Text_Delete_Param /* 109 */,
-   XCF_Scale /* 110 */,		XCF_ChangeStyle /* 111 */,
+   XCF_Graphic	/* 94 */,	XCF_SelectBox /* 95 */,
+   XCF_Connectivity /* 96 */,	XCF_Continue_Element /* 97 */,
+   XCF_Finish_Element /* 98 */, XCF_Continue_Copy /* 99 */,
+   XCF_Finish_Copy /* 100 */,	XCF_Finish /* 101 */,
+   XCF_Cancel_Last /* 102 */,	XCF_Sim /* 103 */,
+   XCF_SPICE /* 104 */,		XCF_PCB /* 105 */,
+   XCF_SPICEflat /* 106 */,	XCF_Rescale /* 107 */,		
+   XCF_Reorder /* 108 */,	XCF_Color /* 109 */,
+   XCF_Text_Delete_Param /* 110 */,
    NUM_FUNCTIONS
 };
 
@@ -1260,6 +1352,7 @@ typedef struct {
    short	coordstyle;
    XPoint	drawingscale;
    XPoint	pagesize;	/* size of page to print on */
+   XPoint	margins;
 } Pagedata;
 
 /*----------------------------------------------------------------------*/
@@ -1289,6 +1382,10 @@ typedef struct _windowdata {
    xcWidget	scrollbarh, scrollbarv;
    Window	window;
    GC 		gc;
+   Pixmap	clipmask;
+   Pixmap	pbuf;		/* clipmask buffer for hierarchical clipping */
+   char		clipped;
+   GC		cmgc;
    int		gccolor, gctype;
    xcIntervalId time_id;
 
@@ -1309,6 +1406,7 @@ typedef struct _windowdata {
    short	filter;		/* selection filter */
    Boolean	manhatn;
    Boolean	boxedit;
+   Boolean	pathedit;
    Boolean	snapto;
    Boolean	bboxon;
    Boolean	center;
@@ -1319,24 +1417,28 @@ typedef struct _windowdata {
    char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
    Boolean	pinpointon;
-   Boolean	attachto;
+   Boolean	pinattach;	/* keep wires attached to pins when moving objinsts */
+   Boolean	showclipmasks;	/* draw clipmask shape outlines */
+#ifndef TCL_WRAPPER
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
+#endif
 #endif
 
    /* buffers and associated variables */
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
+   short	attachto;
    short	lastlibrary;
+   short	textpos;
+   short	textend;
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
    pushlistptr	stack;
    pushlistptr	hierstack;
    short	event_mode;
-   short	editcycle;
-   short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
 } XCWindowData;
@@ -1364,17 +1466,27 @@ typedef struct _undostack {
 /* the library page, and not be destroyed on every call to "composelib"	*/
 
 typedef struct {
-   char		*filename;	/* Last file loaded into the library */
-   char		flags;		/* Flags for library page (changed, read-only) */
    short	number;
    objectptr	*library;
    liblistptr	instlist;	/* List of instances */
 } Library;
 
+typedef struct _Technology *TechPtr;
+
+typedef struct _Technology {
+   u_char	flags;		/* Flags for library page (changed, read-only) */
+   char		*technology;	/* Namespace name (without the "::") */
+   char		*filename;	/* Library file associated with technology */
+   TechPtr 	next;		/* Linked list */
+} Technology;
+
 /* Known flags for library pages */
 
 #define LIBRARY_CHANGED		0x01
 #define LIBRARY_READONLY	0x02
+#define LIBRARY_REPLACE		0x04	/* Replace instances when reading */
+#define LIBRARY_REPLACE_TEMP	0x08	/* Temporary store */
+#define LIBRARY_USED		0x10	/* Temporary marker flag */
 
 /*----------------------------------------------------------------------*/
 /* A convenient structure for holding all the object lists		*/
@@ -1388,10 +1500,11 @@ typedef struct {
    Boolean	retain_backup;
    xcIntervalId	timeout_id;
    int		save_interval;
-   short	do_replace;
    Boolean	filefilter;	/* Is the file list filtered? */
    Boolean	hold;		/* allow HOLD modifiers on buttons */
+   Boolean	showtech;	/* Write technology names in library */
    u_short	new_changes;
+   char		suspend;	/* suspend graphics updates if TRUE */
    short	numlibs;
    short	pages;
    Pagedata	**pagelist;
@@ -1399,6 +1512,7 @@ typedef struct {
    Undoptr	redostack;	/* actions to redo */
    Library	fontlib;
    Library	*userlibs;
+   TechPtr 	technologies;
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
